/*
 * Copyright 1992, 1993, 1994 The Board of Trustees of the Leland Stanford
 * Junior University. Copyright 1994, 1995 Dave Nichols and Steve Cole.
 * Permission is hereby given to use, copy, modify, and
 * distribute this software, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * STANFORD AND THE AUTHORS MAKE NO REPRESENTATIONS OR WARRANTIES OF ANY 
 * KIND CONCERNING  THIS SOFTWARE.
 */
/*
 *
 *  source file:   ./xtpanel/box.c
 *
 * Steve Cole, Dave Nichols (SEP), August 28 1992
 *      Inserted this sample edit history entry.
 *      Please log any further modifications made to this file:
 */

#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>

#include <X11/Xaw/Box.h>
#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Viewport.h>
#include <X11/Xmu/Converters.h>

#include <stdio.h>
#include <assert.h>

#include "object.h"
#include "tree.h"
#include "builders.h"

void  build_box(root,parent,type)
     entry *root;
     Objdef* parent;
     char *type;
{
    Objdef *object;
    Widget parentw, box, form, viewport, message;
    struct compositeinfo *box_info;
    Arg args[40];
    int narg;
    char defname[12];
    static int numbox=1;
    static int numpane=1;
    int height, width;
    extern Cardinal find_insert_position();
    extern void box_destroy();
    extern void box_sync();
    char *tmp;
    
    /* create new object */
    object = new_object(parent);

    parentw=parent->widgetname;

    /* construct default dialog name */
    if (!strcmp(type,"pane") || !strcmp(type,"hpane") || 
	!strcmp(type,"vpane")) { 
    		sprintf(defname,"pane%d",numpane++);
    } else {
    		sprintf(defname,"box%d",numbox++);
    }
    
    /* find name in tree */
    object->name = get_value(root,"name",defname);
    box_info = 
        (struct compositeinfo*) XtMalloc( sizeof( struct compositeinfo ) );
    box_info->topwidget = (Widget) 0;
    box_info->insert_position = (Widget) 0;

    /* height and width are needed later */
    if (is_specified(root,"height")) {
        height = (int) atoi(tmp=get_value(root,"height","0"));XtFree(tmp);
    }
    if (is_specified(root,"width")) {
        width = (int) atoi(tmp=get_value(root,"width","0"));XtFree(tmp);
    }

    narg = 0;
    /* common parameters */
    common_tags(parentw,root,args,&narg,SET_BEFORE);

    /* put inside scrollable viewport if:
     * height specified for vbox, vpane
     * width specified for hbox, hpane */
    if ((is_specified(root,"height") && (!strcmp(type,"box") ||
     !strcmp(type,"vbox") || !strcmp(type,"pane") || !strcmp(type,"vpane")))
     || (is_specified(root,"width") && (!strcmp(type,"hbox") ||
     !strcmp(type,"hpane")))) {

      if (is_specified(root,"height")) {
        XtSetArg(args[narg], XtNheight, height ); narg++;
        if (!strcmp(type,"box") || !strcmp(type,"vbox") || 
		!strcmp(type,"pane") || !strcmp(type,"vpane")) {
          XtSetArg(args[narg], XtNallowVert, True ); narg++;
	}
      }
      if (is_specified(root,"width")) {
        XtSetArg(args[narg], XtNwidth, width ); narg++;
        if (!strcmp(type,"hbox") || !strcmp(type,"hpane")) {
          XtSetArg(args[narg], XtNallowHoriz, True ); narg++;
	}
      }
      XtSetArg(args[narg], XtNresizable, False); narg++;
      assert( narg <40 );
      viewport = XtCreateManagedWidget("viewport",viewportWidgetClass,
                 parentw,args,narg);

      box_info->topwidget = viewport;

      /* a form with a bogus message at the bottom right keeps
       * the viewport from shrinking if its child is initially small */
      narg = 0;
      /* common parameters */
      common_tags(parentw,root,args,&narg,SET_BEFORE);
      XtSetArg(args[narg], XtNborderWidth, 0); narg++;
      XtSetArg(args[narg], XtNresizable, True); narg++;
      assert( narg <40 );
      form = XtCreateManagedWidget("form",formWidgetClass,
                 viewport,args,narg);

      narg = 0;
      /* see explanation of 0.9 in form.c */
      if (is_specified(root,"height")) {
        XtSetArg(args[narg], XtNvertDistance, height*0.9); narg++;
      }
      if (is_specified(root,"width")) {
        XtSetArg(args[narg], XtNhorizDistance, width*0.9); narg++;
      }
      XtSetArg(args[narg], XtNborderWidth, 0); narg++;
      XtSetArg(args[narg], XtNlabel, ""); narg++;
      XtSetArg(args[narg], XtNmappedWhenManaged, False); narg++;
      assert( narg <40 );
      message = XtCreateManagedWidget("message",labelWidgetClass,
                 form,args,narg);

      /* box/paned goes inside form */
      parentw = form;
      narg = 0;
      common_tags(parentw,root,args,&narg,SET_BEFORE);
      /* viewport already has a border */
      XtSetArg(args[narg], XtNborderWidth, 0); narg++;

    }

    /* choose proper orientation */
    if (!strcmp(type,"vbox") || !strcmp(type,"vpane")) {
      XtSetArg(args[narg], XtNorientation, XtorientVertical); narg++;
    }
    if (!strcmp(type,"hbox") || !strcmp(type,"hpane")) {
      XtSetArg(args[narg], XtNorientation, XtorientHorizontal); narg++;
    }

    assert( narg <40 );
    if (!strcmp(type,"pane") || !strcmp(type,"hpane") || 
	!strcmp(type,"vpane")) { 
        box = XtCreateManagedWidget(object->name,panedWidgetClass,
					parentw,args,narg);
    } else {
        box = XtCreateManagedWidget(object->name,boxWidgetClass,
					parentw,args,narg);
    }

    object->widgetname = box;
    object->destroy = box_destroy;
    object->sync = box_sync;
    if (box_info->topwidget == (Widget) 0) box_info->topwidget = box;
    /* this is defined mainly to avoid having height and width
     * applied to all the box children in update_common_tags. */
    object->childwidget = box_info->topwidget;
    object->info = box_info;

    /* create all objects within this box */
    process_tree( root, object );

    /* now hide object if needed */
    /* try setting size here */
    narg = 0;
    common_tags(box,root,args,&narg,SET_AFTER);

    return ;
}

Cardinal find_insert_position(w)
Widget w;
{
     Objdef *object;
     struct compositeinfo *composite_info;
     Arg arg[1];
     Cardinal numchild;
     WidgetList children;
     Cardinal i;

     object = find_by_widget(XtParent(w));
     composite_info = (struct compositeinfo *) object->info;

     /* find out how many children this widget has */
     XtSetArg( arg[0], XtNnumChildren, &numchild );
     XtGetValues( object->widgetname, arg, ONE );

     if (composite_info->insert_position != (Widget) 0)
     {

	/* get a list of the children */
     	XtSetArg( arg[0], XtNchildren, &children );
     	XtGetValues( object->widgetname, arg, ONE );

        /* loop over children, find the right one */
	for (i=0; i<numchild; i++)
	{
		if (children[i] == composite_info->insert_position)
		{
			return ( i+1 );
		}
	}
     } else {
	return ( numchild );
     }

}

void box_sync(object,tag,value)
Objdef* object;
char* tag;
char* value;
{
    Arg args[20];
    int narg;

    /* visible needs to apply to the viewport if there is one */
    if (!strcmp(tag,"visible") && 
  	   XtClass(XtParent(object->widgetname)) == viewportWidgetClass) {
    	narg=0;
    	SetTag(XtParent(object->widgetname),args,&narg,tag,value);
    	XtSetValues(object->widgetname,args,narg);
    } else {
    /* send everything else to update_common_tag */
	update_common_tag(object,tag,value);
    }
}

/* this routine destroys a box */
void box_destroy(object)
Objdef* object;
{
    struct compositeinfo *box_info;

    box_info = (struct compositeinfo*) object->info;

    /* destroy the outermost widget */
         XtDestroyWidget(box_info->topwidget);
}
