
/*
 * Copyright 1992, 1993, 1994 The Board of Trustees of the Leland Stanford
 * Junior University. Copyright 1994, 1995 Dave Nichols and Steve Cole.
 * Permission is hereby given to use, copy, modify, and
 * distribute this software, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * STANFORD AND THE AUTHORS MAKE NO REPRESENTATIONS OR WARRANTIES OF ANY 
 * KIND CONCERNING  THIS SOFTWARE.
 */

/*
 * Glenn Golden, (AT&T Bell Labs), July 3, 1995
 *	Inserted this sample edit history entry.
 *	Please log any further modifications made to this file:
 *
 * Glenn Golden (AT&T Bell Labs), July 3, 1995
 *	Added stuff for sciplot object.
 *
 *  Dave Nichols (Geco-Prakla), Dec 9 1995
 *      Added syntax status of pixmap attribute.
 *

 */
#include <stdio.h>
#include "tree.h"

/* an obj_syn defines an object name and its level 
 * level 0 = composite, must be child of another composite
 * level 1 = basic object, must be child of composite
 * level 2 = subobject, must be child of object
 * level 3 = subsubobject, must be child of subobject
 */
struct obj_syn {  char* name; int level; };

static struct obj_syn objlist[] = {
	{"root",0},
	{"toplevel",0},
	{"window",0},
	{"box",0},
	{"hbox",0},
	{"vbox",0},
	{"pane",0},
	{"hpane",0},
	{"vpane",0},
	{"form",0},
	{"template",0},
	{"button",1},
	{"choice",1},
	{"dialog",1},
	{"field",1},
	{"graph",1},
	{"grid",1},
	{"list",1},
	{"message",1},
	{"menubutton",1},
	{"scrollbar",1},
	{"slider",1},
	{"startup",1},
	{"text",1},
	{"timer",1},
	{"input",1},
	{"output",1},
	{"pipe",1},
	{"toggle",1},
	{"togglegrid",1},
	{"sciplot",1},		/* Added GDG 7/3/95 */
	{"var",1},
	{"item",2},
	{"itemlist",2},
	{"test",3},
	{"false",3},
	{"true",3},
};

static int nobj = sizeof(objlist)/sizeof(objlist[0]);

/* an attr_syn defines an attribute name and its level 
 * An attribute must have a level less than or equal to
 * its parent.
 *
 * e.g. if an attribute is valid for objects but not composites the
 *      make it level 1.
 */
struct attr_syn {  char* name; int level; };

static struct attr_syn attrlist[] = {
	{"name",0},
	{"value",0},
	{"label",0},
	{"height",0},
	{"width",0},
	{"borderWidth",0},
	{"foreground",0},
	{"background",0},
	{"borderColor",0},
	{"backgroundBitmap",0},
	{"backgroundPixmap",0},
	{"sensitive",0},
	{"x",0},
	{"y",0},
	{"fromHoriz",0},
	{"fromVert",0},
	{"hSpace",0},
	{"vSpace",0},
	{"visible",0},
	{"orientation",0},
	{"bitmap",1},
	{"pixmap",1},
	{"editType",1},
	{"font",1},
	{"tab",1},
	{"state",1},
	{"action",1},
	{"interval",1},
	{"running",1},
	{"format",1},
	{"columns",1},
	{"formtype",1},
	{"onvalue",1},
	{"offvalue",1},
	{"position",1},
	{"nsamp",1},
	{"nx",1},
	{"ny",1},
	{"xmin",1},
	{"ymin",1},
	{"dx",1},
	{"dy",1},
	{"min",1},
	{"max",1},
	{"stream",1},
	{"separator",1},
	{"plottype", 1},	/* SciPlot plot type. */
	{"xlabel",1},		/* SciPlot x axis label. */
	{"ylabel",1},		/* SciPlot y axis label. */
	{"linestyle",1},	/* SciPlot line style. */
	{"marker",1},		/* SciPlot point marker type. */
	{"list",2},
	{"condition",2},
};

static int nattr = sizeof(attrlist)/sizeof(attrlist[0]);

/* routine to check validity of relationship of objects and subobjects */

extern int is_template();

int bad_obj_syntax( parent, child )
entry* parent;
entry* child;
{
int i, ip, ic ;


ip= -1;
for( i=0; i<nobj ; i++ ) { 
    if( !strcmp( parent->tag, objlist[i].name ) ){
       ip=i; break; } 
}
if( ip == -1 ) ip=0;

ic= -1;
for( i=0; i<nobj ; i++ ) { 
    if( !strcmp( child->tag, objlist[i].name ) ){
       ic=i; break; } 
}

if( is_template( child->tag ) ) ic=0;

if( ic== -1 ) {
	fprintf(stderr,"Unknown object name \"%s\"\n",child->tag);
	return 1;
}

if( objlist[ic].level == 0 ){
   if( objlist[ip].level != 0 ){
      fprintf(stderr,"Object \"%s\" can not be a child of \"%s\" \n", 
		child->tag,parent->tag);
	return 1;
   }
}else{
   if( (objlist[ic].level <= objlist[ip].level) &&
	(objlist[ic].level != 3 &&  objlist[ip].level != 3)){
	fprintf(stderr,"Object \"%s\" can not be a child of \"%s\" \n",
				child->tag,parent->tag);
	return 1;
   }

}


return 0;

}

int bad_attr_syntax( parent, child )
entry* parent;
entry* child;
{

int i, ip, ic ;

ip= -1;
for( i=0; i<nobj ; i++ ) { 
    if( !strcmp( parent->tag, objlist[i].name ) ){
       ip=i; break; } 
}
if( ip == -1 ) ip=0;

ic= -1;
for( i=0; i<nattr ; i++ ) { 
    if( !strcmp( child->tag, attrlist[i].name ) ){
       ic=i; break; } 
}
if( is_template( parent->tag ) ) ic=0;
if( ic== -1 ) {
	fprintf(stderr,"Unknown attribute name \"%s\"\n",child->tag);
	return 1;
}

if( attrlist[ic].level > objlist[ip].level ){
	fprintf(stderr,"Attribute \"%s\" is not valid for an object of type \"%s\" \n",
				child->tag,parent->tag);
	return 1;
}


return 0;
}

