/*
 * Copyright 1992, 1993, 1994 The Board of Trustees of the Leland Stanford
 * Junior University. Copyright 1994, 1995 Dave Nichols and Steve Cole.
 * Permission is hereby given to use, copy, modify, and
 * distribute this software, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * STANFORD AND THE AUTHORS MAKE NO REPRESENTATIONS OR WARRANTIES OF ANY 
 * KIND CONCERNING  THIS SOFTWARE.
 */

/*
 *
 *  source file:   ./xtpanel/choice.c
 *
 * Steve Cole, Dave Nichols (SEP), August 28 1992
 *      Inserted this sample edit history entry.
 *      Please log any further modifications made to this file:
 * Dave Nichols (SEP), June 8 1993
 *      Don't re-expand items that come from a list.
 * Steve Cole (SEP), October 8 1993
 *      Added default separator.
 * Dave Nichols (SEP), May 26 1994
 *      Add per-item actions.
 * Dave Nichols (GecoPrakla), Jan 10 1996
 *      Add destroy proc to free up items.
 */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <X11/Xaw/Box.h>
#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Toggle.h>
#include <assert.h>

#include <stdio.h>


#include "object.h" 
#include "item.h"
#include "tree.h"
#include "builders.h"

typedef struct choicedef {
    char* value;
    Widget widgetname;
    Objdef* object;
    struct choicedef* next;
} _cdef;

/* translation table used for toggle widgets in a choice object */
char choice_trans[] =
  "<EnterWindow>:   highlight(Always)\n\
   <LeaveWindow>:   unhighlight()\n\
   <Btn1Down>,<Btn1Up>:   set() notify()";

extern void choice_destroy();

void build_choice(root,parent)
     entry *root;
     Objdef* parent;
{
    Objdef *object;
    entry *curr;
    struct itemdef *item;
    struct itemizedinfo *choice_info;
    char *label,*value,*fromlist;
    Arg args[20];
    int narg;
    Widget box,labl,choice1,choice2;
    char defname[12];
    static int numtog=1;
    int nitem;
    extern void choice_callback();
    extern void choice_update();
    
    narg = 0;
    /* common parameters */
    common_tags(parent->widgetname,root,args,&narg,SET_BEFORE);

    /* choice object gets its own box */
    assert( narg < 20 );
    box = XtCreateManagedWidget("choicebox",boxWidgetClass,
	parent->widgetname,args,narg);
    
    /* create new object */
    object = new_object(parent);

    /* construct default button name */
    sprintf(defname,"choice%d",numtog++);
    
    /* find label, name, action in tree */
    object->name = get_value(root,"name",defname);
    label = get_value(root,"label",object->name);
    object->action = parse_actions(object->name,root);
    
    choice_info = (struct itemizedinfo*) 
                   XtMalloc( sizeof( struct itemizedinfo));
    choice_info->firstitem = (struct itemdef*) 0;
    /* define a default separator */
    choice_info->separator = XtNewString(" \n");
    object->info = choice_info;

    /* make the label */
    /* if label is not specified, skip displaying it */
    if (is_specified(root,"label")) {
	narg = 0;
     	XtSetArg(args[narg], XtNborderWidth, 0); narg++;
     	XtSetArg(args[narg], XtNlabel, label); narg++;
    	/* common parameters */
    	common_tags(box,root,args,&narg,SET_BEFORE & ~SET_SIZE);

     	labl = XtCreateManagedWidget(object->name,labelWidgetClass,box,
				     args,narg);
    }
    
    /* first parse any itemlist entries into items */
    parse_itemlist( root, &(choice_info->separator) );
    
    /* loop over items */
    for (curr = get_next(root,"item",(entry*) 0), nitem=0; 
	 curr != ((entry*) 0); 
	 curr = get_next(root,"item",curr), nitem++){
	
	/* construct default item name */
	sprintf(defname,"item%d",nitem+1);
	
	/* allocate structure to hold info for choice item */
	item = new_item(choice_info);
	
	/* get the fromlist attribute to see if we want to expand the label */
        /* if it came from a list we don't want to expand it again */
        fromlist = get_value( curr,"fromlist","false");

        /* find label, value in the next level of the tree */
	XtFree(label);
        if( !strcmp( fromlist, "false" ) ){
           label = get_value(curr,"label",defname);
        }else{
           label = get_raw_value(curr,"label",defname);
        }
        item->value = get_value(curr,"value",label);
        XtFree( fromlist );
	
	/* value is default; we set a button to on if its value matches this */
	if (nitem == 0) value = get_value(root,"value",item->value);

        /* see if this item has any actions of its own */       
        item->item_actions = parse_actions(object->name,curr);
	
	/* check to see if this is the default item */
	/* choice object always needs to have one button set */
	narg = 0;
	if (!strcmp(value,item->value)) {
	    XtSetArg(args[narg], XtNstate, True); narg++;
	    object->value = value;
	}
	
	/* first button stands alone; others have to point to it 
	   with radioGroup */
	if (nitem == 0) {
	    XtSetArg(args[narg], XtNlabel, label); narg++;
	    /* radioData is used by choice_update routine */
	    XtSetArg(args[narg], XtNradioData, item->value); narg++;
	    /* common parameters */
    	    common_tags(box,curr,args,&narg,SET_BEFORE);

            assert( narg < 20 );
	    choice1 = XtCreateManagedWidget(object->name,toggleWidgetClass,box,
					    args,narg);
            /* use translations */
    	    XtOverrideTranslations(choice1,
                   XtParseTranslationTable(choice_trans));
	    XtAddCallback(choice1, XtNcallback,choice_callback, 
			  (XtPointer)item );
	    item->widgetname = choice1;
	}
	else {
	    XtSetArg(args[narg], XtNradioGroup, choice1); narg++;
	    XtSetArg(args[narg], XtNlabel, label); narg++;
	    XtSetArg(args[narg], XtNradioData, item->value); narg++;
	    /* common parameters */
    	    common_tags(box,curr,args,&narg,SET_BEFORE);

            assert( narg < 20 );
	    choice2 = XtCreateManagedWidget(object->name,toggleWidgetClass,
                         box,args,narg);

            /* use translations */
    	    XtOverrideTranslations(choice2,
                   XtParseTranslationTable(choice_trans));
	    XtAddCallback(choice2, XtNcallback,choice_callback, 
			  (XtPointer)item );
	    item->widgetname = choice2;
	}
	
	item->object = object;
	item->label = XtNewString(label);
        if (nitem == 0) choice_info->firstitem = item;


    }

    XtFree(label);
    object->widgetname = box;
    object->updater = choice_update;
    object->destroy = choice_destroy;

    /* now hide object if needed */
    narg = 0;
    common_tags(box,root,args,&narg,SET_AFTER);

}

/*
 * choice_callback
 * called when one of the choices is selected 
 */
void choice_callback(widget, client_data, callData)
     Widget widget;
     XtPointer client_data, callData;
{
    Objdef *object;
    struct itemdef *item;
    Arg arg[1];
    Boolean state;
    
    item = (struct itemdef *) client_data;
    object = (Objdef *) item->object;
    /* get current state of button */
    XtSetArg( arg[0], XtNstate, &state );
    XtGetValues( item->widgetname, arg, ONE );
    /* if button is now selected, perform its action */
    if (state == TRUE) 
      {
	  XtFree( object->value );
	  object->value = XtNewString((char *) item->value);
	  /* do any item specific actions or object actions */
          if( item->item_actions != 0 && item->item_actions->type != NONE ){
              perform_actions(object->name,item->item_actions,1);
          }else{
	      perform_actions(object->name,object->action,1);
	  }
      }
}

/*
 * choice_update
 * called when a choice value is changed by an ASSIGN action
 */
void choice_update(object,value)
Objdef *object;
char *value;
{
    struct itemdef *item;
    struct itemizedinfo *choice_info;
    Arg arg[1];
    int nitem;

    choice_info = (struct itemizedinfo *) object->info;

    /* loop over items - look for one that matches value and highlight */
    for (item = choice_info->firstitem,nitem=0;
	item != (struct itemdef*) 0; item=item->next,nitem++) {
	if (!strcmp(item->value,value)) {
    		XawToggleUnsetCurrent(item->widgetname);
    		XtSetArg( arg[0], XtNstate, TRUE );
    		XtSetValues( item->widgetname, arg, ONE );
		break;
	}
    }
}

void choice_destroy(object)
Objdef* object;
{
    struct itemdef *next;
    struct itemdef *item;
    struct itemizedinfo *choice_info;
    int nitem;

    choice_info = (struct itemizedinfo *) object->info;
    for (item = choice_info->firstitem;
	item != (struct itemdef*) 0; item=next ) {
	XtFree( item->label );
	XtFree( item->value );
	if( item->item_actions !=0 ) free_action( item->item_actions );
        next = item->next;
	XtFree( item );
    }

    XtFree( choice_info->separator );


}
