/*
 * Copyright 1992, 1993, 1994 The Board of Trustees of the Leland Stanford
 * Junior University. Copyright 1994, 1995 Dave Nichols and Steve Cole.
 * Permission is hereby given to use, copy, modify, and
 * distribute this software, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * STANFORD AND THE AUTHORS MAKE NO REPRESENTATIONS OR WARRANTIES OF ANY 
 * KIND CONCERNING  THIS SOFTWARE.
 */

/*
 *
 *  source file:   ./xtpanel/dialog.c
 *
 * Steve Cole, Dave Nichols (SEP), August 28 1992
 *      Inserted this sample edit history entry.
 *      Please log any further modifications made to this file:
 * Steve Cole (SEP), February 23 1993
 *      Ok button suppressed if all actions are STRING or NONE.
 * Glenn Golden (Bell Labs, Lucent Technologies), Apr 21, 1996
 *	Avoid XtNewString() calls with XawDialogGetValueString() 
 *      as an arg; XtNewString() is a macro and passing it a
 *      function loses if macro not written with this in mind.
 *      
 */


#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Dialog.h>
#include <X11/Xaw/AsciiText.h>

#include <stdio.h>
#include <assert.h>

#include "object.h" 
#include "tree.h"
#include "builders.h"

/* translation table used for dialog widget */
char trans[] = 		
  "<Leave>:         update_dialog()\n\
   <Key>Return:     return_key()";

void build_dialog(root,parent)
     entry *root;
     Objdef* parent;
{
    Objdef *object;
    char *label;
    Arg args[20];
    Widget dialog, button;
    char defname[12];
    static int numdiag=1;
    int narg;
    Cardinal numchild;
    WidgetList children;
    Cardinal i;
    extern void dialog_callback();
    extern void dialog_update();
    
    /* create new object */
    object = new_object(parent);

    /* construct default dialog name */
    sprintf(defname,"dialog%d",numdiag++);
    
    /* find label, name, action, etc. in tree */
    object->name = get_value(root,"name",defname);
    label = get_value(root,"label",object->name);
    object->action = parse_actions(object->name,root);
    object->value = get_value(root,"value","");
    
    narg=0;
    XtSetArg(args[narg], XtNlabel, label ); narg++;
    XtSetArg(args[narg], XtNvalue, object->value); narg++;
    /* common parameters */
    common_tags(parent->widgetname,root,args,&narg,SET_BEFORE);

    assert ( narg < 20 );
    dialog = XtCreateManagedWidget(object->name,dialogWidgetClass,
	parent->widgetname,args,narg);

    object->widgetname = dialog;

    /* now hide object if needed */
    narg = 0;
    common_tags(dialog,root,args,&narg,SET_AFTER);

    /* have to go down inside the dialog widget to find the
     * text entry field */
    narg = 0;
    XtSetArg( args[narg], XtNnumChildren, &numchild ); narg++;
    XtSetArg( args[narg], XtNchildren, &children ); narg++;
    XtGetValues( dialog, args, narg );
    /* loop over children */
    for (i=0; i<numchild; i++)
    {
	if (XtClass(children[i]) == asciiTextWidgetClass)
		object->childwidget = children[i];
    }

    object->updater = dialog_update;
    
    /* use translations to update value whenever mouse leaves dialog widget */
    XtOverrideTranslations(dialog,XtParseTranslationTable(trans));
    
    /* add a button */
    if (!all_string_actions(object->action)) {
      button = XtCreateManagedWidget(object->name,commandWidgetClass,
				       dialog,NULL,ZERO);
      XtAddCallback(button, XtNcallback, dialog_callback, 
		      (XtPointer) object );
    }

    XtFree(label);
}

/* 
 * this routine gets called when the optional button of a dialog
 * item is pressed 
 */
void  dialog_callback(widget, client_data, callData)
     Widget widget;
     XtPointer client_data, callData;
{
    Objdef *object;
    char	*wstr;		/* String from the widget. */
    
    object = (Objdef *) client_data;
    /* get the value of the dialog */
    if( object->value != 0 ) XtFree( object->value );
    /*
     * XtNewString() may be a macro, better not invoke it
     * with a function as an arg, so do it indirectly thru wstr.
     *
     *		- gdg
     */
    wstr = XawDialogGetValueString(object->widgetname);
    object->value = XtNewString(wstr);
    perform_actions(object->name,object->action,1);
}

/*
 * this routine gets called whenever we leave
 * a dialog widget, to be sure its value is up to date
 */
void dialog_update_callback( w, event, params, num_params)
     Widget w;
     XEvent *event;
     String *params;
     Cardinal *num_params;
{
    Objdef* obj;
    char	*wstr;		/* String from the widget. */
    
    obj = find_by_widget(w);  
    if( obj->value != 0 ) XtFree( obj->value );
    wstr = XawDialogGetValueString(w);
    obj->value = XtNewString(wstr);
}

/*
 * this routine gets called whenever we press return
 * in a dialog widget.
 */
void return_key_callback( w, event, params, num_params)
     Widget w;
     XEvent *event;
     String *params;
     Cardinal *num_params;
{
    Objdef *object;
    Widget dialog = XtParent(w);
    char	*wstr;		/* String from the widget. */
   
    object = find_by_widget(dialog);  
    /* get the value of the dialog */
    if( object->value != 0 ) XtFree( object->value );
    wstr = XawDialogGetValueString(object->widgetname);
    object->value = XtNewString(wstr);

    perform_actions(object->name,object->action,1);
    return;
}

/*
 * this routine gets called when a dialog's value is updated
 * by an ASSIGN action.
 */
void dialog_update(object,value)
Objdef *object;
char *value;
{
    Arg args[20];
    int narg;
    
    narg=0;
    XtSetArg(args[narg], XtNvalue, value); narg++;
    XtSetValues(object->widgetname,args,narg);
}
