/*
 * Copyright 1992, 1993, 1994 The Board of Trustees of the Leland Stanford
 * Junior University. Copyright 1994, 1995 Dave Nichols and Steve Cole.
 * Permission is hereby given to use, copy, modify, and
 * distribute this software, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * STANFORD AND THE AUTHORS MAKE NO REPRESENTATIONS OR WARRANTIES OF ANY 
 * KIND CONCERNING  THIS SOFTWARE.
 */


/*
 *
 *  source file:   ./xtpanel/eval_command.c
 *
 * Steve Cole, Dave Nichols (SEP), September 9 1992
 *      Inserted this sample edit history entry.
 *      Please log any further modifications made to this file:
 */

/* This routine should evaluate a command using an appropriate shell
 * and return the data from stdout in the a character buffer.
 *
 * This implementation uses popen() to do this. There may be faster ways on
 * some systems but this is reasonably portable.
 */

#include <stdio.h>
#include <string.h>

#include <X11/Xos.h>

#include "eval_command.h"
#include "string_buf.h"

/* evaluate a back-quoted command */
#define CBUF_LEN 2000
static char comm_buf[CBUF_LEN+1];

char* eval_command( comm )
char* comm;
{
FILE* in;
int i,num;
char *ret;
string_buf *buffer;

if( (in = popen( comm, "r" )) == (FILE*)0 ) {
   perror("eval_command"); 
   exit(-1);
}

buffer = buf_start();

while( (num = fread( comm_buf, 1, CBUF_LEN, in )) != 0 ){
    /* check for embedded nulls */
    for( i=0 ; i<num; i++ ) {
      if( comm_buf[i] == '\0' ){
        fprintf(stderr,"Embedded null found in output of command \"%s\"\n", comm );
        buf_cat( buffer, comm_buf, i );   
	ret = buf_fetch( buffer );
        fprintf(stderr,"Output up to this point is shown between ---start--- and ---end--- \n---start---\n%s\n---end---\n", ret );
        fprintf(stderr,"Pipe the output of your command through \"cat -v\" to avoid this \n");
        exit(-1);
      }
    }
    buf_cat( buffer, comm_buf, num );   
};
pclose( in );

/* trim off trailing whitespace */
buf_trim( buffer, " \t\n" );

ret = buf_fetch( buffer );


buf_free( buffer );

return ret;
}
