/*
 * Copyright 1992, 1993, 1994 The Board of Trustees of the Leland Stanford
 * Junior University. Copyright 1994, 1995 Dave Nichols and Steve Cole.
 * Permission is hereby given to use, copy, modify, and
 * distribute this software, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * STANFORD AND THE AUTHORS MAKE NO REPRESENTATIONS OR WARRANTIES OF ANY 
 * KIND CONCERNING  THIS SOFTWARE.
 */

 
/*
 *
 *  source file:   ./xtpanel/form.c
 *
 * Steve Cole, Dave Nichols (SEP), August 28 1992
 *      Inserted this sample edit history entry.
 *      Please log any further modifications made to this file:
 */

#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>

#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Viewport.h>
#include <X11/Xmu/Converters.h>

#include <stdio.h>
#include <assert.h>


#include "object.h"
#include "tree.h"
#include "builders.h"

void build_form(root,parent,type)
     entry *root;
     Objdef* parent;
     char *type;
{
    Objdef *object;
    Widget form, viewport, parentw, dummy;
    struct compositeinfo *form_info;
    Arg args[20];
    int narg;
    char defname[12];
    static int numform=1;
    int height, width;
    extern void box_sync();
    char *tmp;

    parentw = parent->widgetname; 

    /* create new object */
    object = new_object(parent);

    /* construct default dialog name */
    sprintf(defname,"form%d",numform++);
    
    /* find name in tree */
    object->name = get_value(root,"name",defname);
    form_info = (struct compositeinfo*) 
	XtMalloc( sizeof( struct compositeinfo ) );

    narg = 0;
    /* common parameters */
    common_tags(parentw,root,args,&narg,SET_BEFORE);

    /* if height/width specified, put form inside a scrollable viewport */
    if (is_specified(root,"height") || is_specified(root,"width")) {
      if (is_specified(root,"height")) {
          XtSetArg(args[narg], XtNallowVert, True ); narg++;
      }
      if (is_specified(root,"width")) {
          XtSetArg(args[narg], XtNallowHoriz, True ); narg++;
      }
      XtSetArg(args[narg], XtNresizable, False); narg++;
      assert( narg < 20 );
      viewport = XtCreateManagedWidget(object->name,viewportWidgetClass,
                 parentw,args,narg);
      parentw = viewport;
      form_info->topwidget = viewport;

      narg = 0;
      /* form inside viewport, do not set height or width */
      common_tags(parentw,root,args,&narg,SET_BEFORE & ~SET_SIZE);
    }

    assert( narg < 20 );
    form = XtCreateWidget(object->name,formWidgetClass,
				parentw,args,narg);

    object->widgetname = form;
    if (form_info->topwidget == (Widget) 0) form_info->topwidget = form;
    object->childwidget = form_info->topwidget;
    /* use sync routine for box to correctly handle visible */
    object->sync = box_sync;
    object->info = form_info;

    /* now hide object if needed */
    narg = 0;
    common_tags(form,root,args,&narg,SET_AFTER);

    /* the form (and viewport) will shrink to just big enough
     * to contain the items. So we create an invisible message
     * at the lower right corner if height or width is specified */
    narg = 0;
    if (is_specified(root,"height") || is_specified(root,"width")) {
      if (is_specified(root,"height")) {
	  height = (int) atoi(tmp=get_value(root,"height","0"));XtFree(tmp);
	  /* the coordinates are for the upper left corner of the
	   * message. If we use height and width, the form will
	   * be slightly too large because of the extra width and height
	   * of the message. 0.9 is a hack; better would be to
	   * compute the message size and subtract it off. */
          XtSetArg(args[narg], XtNvertDistance, height*0.9); narg++;
      }
      if (is_specified(root,"width")) {
	  width = (int) atoi(tmp=get_value(root,"width","0"));XtFree(tmp);
          XtSetArg(args[narg], XtNhorizDistance, width*0.9); narg++;
      }
      XtSetArg(args[narg], XtNlabel, ""); narg++;
      XtSetArg(args[narg], XtNborderWidth, 0); narg++;
      XtSetArg(args[narg], XtNmappedWhenManaged, False); narg++;
      assert( narg < 20 );
      dummy = XtCreateManagedWidget("dummy",labelWidgetClass,
                 form,args,narg);
    }

    /* also create a bogus message in upper left corner.
     * otherwise if an empty form is created, Xt gets upset
     * because it has zero height and width */
    narg = 0;
    XtSetArg(args[narg], XtNvertDistance, 0); narg++;
    XtSetArg(args[narg], XtNhorizDistance, 0); narg++;
    XtSetArg(args[narg], XtNlabel, ""); narg++;
    XtSetArg(args[narg], XtNborderWidth, 0); narg++;
    XtSetArg(args[narg], XtNmappedWhenManaged, False); narg++;
    assert( narg < 20 );
    dummy = XtCreateManagedWidget("dummy",labelWidgetClass,
                form,args,narg);

    /* create all objects within this form */
    process_tree( root, object );

    /* now manage the form. Otherwise if we manage it at creation
     * time and the form is empty, Xt complains. */
    XtManageChild(form);

    return ;
}
