/*
 * Copyright 1992, 1993, 1994 The Board of Trustees of the Leland Stanford
 * Junior University. Copyright 1994, 1995 Dave Nichols and Steve Cole.
 * Permission is hereby given to use, copy, modify, and
 * distribute this software, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * STANFORD AND THE AUTHORS MAKE NO REPRESENTATIONS OR WARRANTIES OF ANY 
 * KIND CONCERNING  THIS SOFTWARE.
 */


/*
 *
 *  source file:   ./xtpanel/item.c
 *
 * Steve Cole, Dave Nichols (SEP), August 31 1992
 *      Inserted this sample edit history entry.
 *      Please log any further modifications made to this file:
 */

#include <stdio.h>

#include <X11/Intrinsic.h>

#include "object.h"
#include "item.h"

/*
 * Function name: new_item
 * Description: creates a structure for a new list
 *	        or menubutton item.
 * Arguments: info - info structure of choice, list, or menubutton.
 * Returns: pointer to new structure of type itemdef
 */
struct itemdef* new_item(info)
     struct itemizedinfo* info;
{
    struct itemdef *a, *b;

    a = (struct itemdef*) XtMalloc( sizeof( struct itemdef ) );
    a->label = (char*)0;
    a->value = (char*)0;
    a->widgetname = (Widget)0;
    a->object = (Objdef*)0;
    a->item_actions = 0;
    a->next = (struct itemdef*)0;

    if( info->firstitem == 0 ){
	info->firstitem = a;
    }else{
    
        /* find the tail of the list and insert a */
        for( b=info->firstitem; b->next != (struct itemdef*)0; b = b->next );
        b->next = a;
    }
    
    return a;
}
/*
 * Function name: find_item
 * Description: loop over all items for a menubutton or list.
 *		find the one with the right label.
 * Arguments: obj - object structure of menubutton or list.
 *	      label - label to match	
 * Returns: pointer to item structure for matching item
 */
struct itemdef*
  find_item(obj,label)
Objdef* obj;
char* label;
{
    struct itemdef* item;
    
    /* loop over items */
    for ( item=((struct itemizedinfo *) obj->info)->firstitem; 
	 item != (struct itemdef*) 0; item=item->next )
      {
	  if (!strcmp(item->label,label)) {
	      return item;
	  }
      }
    
    fprintf(stderr,"find_item cannot find item with label %s\n",
	    (char *) label);
    exit(-1);  
}
