/*
 * Copyright 1992, 1993, 1994 The Board of Trustees of the Leland Stanford
 * Junior University. Copyright 1994, 1995 Dave Nichols and Steve Cole.
 * Permission is hereby given to use, copy, modify, and
 * distribute this software, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * STANFORD AND THE AUTHORS MAKE NO REPRESENTATIONS OR WARRANTIES OF ANY 
 * KIND CONCERNING  THIS SOFTWARE.
 */


/*
 *
 *  source file:   ./xtpanel/list.c
 *
 * Steve Cole, Dave Nichols (SEP), August 28 1992
 *      Inserted this sample edit history entry.
 *      Please log any further modifications made to this file:
 * Steve Cole (SEP), February 25 1993
 *      Added number of columns. Changed list to use form widget.
 * Steve Cole (SEP), May 10 1993
 *      List separator is stored in info structure for SetTag.
 * Dave Nichols (SEP), June 8 1993
 *      Don't re-expand items that come from an itemlist.
 * Steve Cole (SEP), October 8 1993
 *      Added default separator.
 * Dave Nichols (SEP), May 26 1994
 *      Add per-item actions.
 * Dave Nichols (Geco-Prakla), Jan 10 1996
 *      Cleaned up some memory leaks.
 * Dave Nichols (Geco-Prakla), Jan 15 1996
 *      Cleaned one more memory leak (value).
 */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <X11/Xaw/Box.h>
#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/List.h>
#include <X11/Xaw/Viewport.h>

#include <stdio.h>
#include <assert.h>


#include "object.h" 
#include "item.h" 
#include "tree.h"
#include "builders.h"

void build_list(root,parent)
     entry *root;
     Objdef* parent;
{
    Objdef *object;
    entry *curr;
    char *label, *value, *fromlist;
    struct itemdef *item;
    struct itemizedinfo *list_info;
    Arg args[20];
    int narg;
    Widget form,labl,viewport,list;
    char defname[12];
    static int numlist=1;
    int nitem=0;
    String * liststring = NULL; 
    int allocated_items = 0;
    int ncols;
    extern void list_callback();
    extern void list_update();
    
    /* list gets its own box - actually it is a form */
    narg = 0;
    /* common parameters */
    common_tags(parent->widgetname,root,args,&narg,SET_POSITION);
    /*
     * the form is not managed. Xt would complain at this
     * point if we used XtCreateManagedWidget and the form was
     * a dynamically created child of another form.
     */
    form = XtCreateWidget("listbox",formWidgetClass,
		parent->widgetname,args,narg);
    
    /* create new object */
    object = new_object(parent);

    /* construct default list name */
    sprintf(defname,"list%d",numlist++);
    
    /* find label, name, action in tree */
    object->name = get_value(root,"name",defname);
    label = get_value(root,"label",object->name);
    object->action = parse_actions(object->name,root);
    list_info = (struct itemizedinfo*) 
            XtMalloc( sizeof( struct itemizedinfo ) );
    list_info->firstitem = (struct itemdef*) 0;
    list_info->listwidget = 0;
    /* define a default separator */
    list_info->separator = XtNewString(" \n");
    object->info = list_info;
    
    /* the list label */
    narg = 0;
    common_tags(parent->widgetname,root,args,&narg,SET_FONT);
    XtSetArg(args[narg], XtNborderWidth, 0); narg++;
    XtSetArg(args[narg], XtNlabel, label); narg++;
    labl = XtCreateManagedWidget(object->name,labelWidgetClass,
                  form,args,narg);
    XtFree( label );
    
    /* the list goes in a viewport to allow scrolling */
    /* the viewport is not resizable */
    narg=0;
    XtSetArg(args[narg], XtNallowHoriz, True ); narg++;
    XtSetArg(args[narg], XtNallowVert, True ); narg++;
    XtSetArg(args[narg], XtNfromVert, labl ); narg++;
    XtSetArg(args[narg], XtNresizable, False); narg++;
    /* common parameters */
    common_tags(parent->widgetname,root,args,&narg,
	SET_BEFORE & ~SET_POSITION);

    /*
     * the viewport is not managed. Xt would complain at this
     * point if we used XtCreateManagedWidget and the list was
     * a dynamically created child of a form.
     */
    assert( narg < 20 );
    viewport = XtCreateWidget( object->name,viewportWidgetClass,
				     form,args,narg);

    /* the list is resizable to allow for scrolling */
    narg=0;
    XtSetArg(args[narg], XtNresizable, True); narg++;
    /* common parameters */
    common_tags(viewport,root,args,&narg,SET_COLOR | SET_FONT | SET_PIXMAP);

    /* number of columns */
    if (is_specified(root,"columns"))
    {
	value = get_value(root,"columns","0");
	sscanf( value,"%d",&ncols);
	XtFree(value);
        XtSetArg(args[narg], XtNforceColumns, TRUE ); narg++;
        XtSetArg(args[narg], XtNdefaultColumns, ncols ); narg++;
    }

    /* now create the list */
    assert( narg < 20 );
    list = XtCreateManagedWidget(object->name,listWidgetClass,viewport,
				 args,narg);
    
    /* first parse any itemlist entries into items */
    parse_itemlist( root, &(list_info->separator) );

    /* loop over again to get items */
    for (curr = get_next(root,"item",(entry*) 0), nitem=0; 
	 curr != ((entry*) 0); 
	 curr = get_next(root,"item",curr), nitem++){
	
	/* construct default item name */
	sprintf(defname,"item%d",nitem+1);
	
	/* allocate structure to hold info for list item */
	item =  new_item(list_info);

	/* get the fromlist attribute to see if we want to expand the label */
        /* if it came from a list we don't want to expand it again */
	fromlist = get_value( curr,"fromlist","false");
        	
	/* find label, value in the next level of the tree */
	if( !strcmp( fromlist, "false" ) ){
	   label = get_value(curr,"label",defname);
	}else{
	   label = get_raw_value(curr,"label",defname);
	}
	XtFree( fromlist );
	item->value = get_value(curr,"value",label);

        /* see if this item has any actions of its own */       
        item->item_actions = parse_actions(object->name,curr);

	
	/* add an item */
	if (nitem == allocated_items) {
	    allocated_items += 5;
	    liststring = (String *) XtRealloc((char *)liststring, 
					   sizeof(String) * allocated_items);
	}
	liststring[nitem] = XtNewString(label);
	XawListChange(list, liststring, nitem+1, 0, True);
	
	item->label = XtNewString(label);
	item->widgetname = list;
	/* callback routine needs to know the object name */
	item->object = object;
	
	/* info structure points to first item for find_item routine */
	if (nitem == 0) list_info->firstitem = item;
	
	/* value is default; we select the button whose value matches this */
	if (nitem == 0) value = get_value(root,"value",item->value);

	XtFree( label );
	
    }
    
    /* now manage the viewport and form */
    XtManageChild(viewport);
    XtManageChild(form);

    /* loop over items - look for one that matches the default */
    for (item = list_info->firstitem,nitem=0; 
         item != (struct itemdef*) 0; item=item->next,nitem++) {
        if (!strcmp(item->value,value)) {
    	    XawListHighlight(list,nitem);
            object->value = value;
        }
    }
    
    XtAddCallback( list, XtNcallback, list_callback, (XtPointer) object );

    list_info->listwidget = list;
    object->widgetname = form;
    object->childwidget = viewport;
    object->updater = list_update;

    /* now hide object if needed */
    narg = 0;
    common_tags(form,root,args,&narg,SET_AFTER);
}

void
  list_callback(widget, client_data, callData)
Widget widget;
XtPointer client_data, callData;
{
    Objdef *object;
    XawListReturnStruct *item;
    struct itemdef *listitem;
    
    object = (Objdef *) client_data;
    /* XawListReturnStruct just tells us the label */
    item = (XawListReturnStruct *) callData;
    
    /* find the item whose label matches */
    listitem = find_item(object,item->string);
    
    /* get the value of the text field */
    if( object->value != 0 ) XtFree(object->value);
    object->value = XtNewString((char *) listitem->value);
    /* do any item specific actions or the object action*/
    if( listitem->item_actions != 0 && listitem->item_actions->type != NONE ){
        perform_actions(object->name,listitem->item_actions,1);
    }else{   
        perform_actions(object->name,object->action,1);
    }
}

void list_update(object,value)
Objdef *object;
char *value;
{
    struct itemdef *item;
    struct itemizedinfo *list_info;
    int nitem=0;
    
    list_info = object->info;

    /* loop over items - look for one that matches the value */
    for (item = list_info->firstitem,nitem=0; 
	     item != (struct itemdef*) 0; item=item->next,nitem++) {
	    if (!strcmp(item->value,value)) {
		XawListHighlight(item->widgetname,nitem);
       		object->value = XtNewString(value);
		break;
	    }
    }
    
}
