/*
 * Copyright 1992, 1993, 1994 The Board of Trustees of the Leland Stanford
 * Junior University. Copyright 1994, 1995 Dave Nichols and Steve Cole.
 * Permission is hereby given to use, copy, modify, and
 * distribute this software, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * STANFORD AND THE AUTHORS MAKE NO REPRESENTATIONS OR WARRANTIES OF ANY 
 * KIND CONCERNING  THIS SOFTWARE.
 */


/*
 *
 *  source file:   ./xtpanel/slider.c
 *
 * Steve Cole, Dave Nichols (SEP), August 28 1992
 *      Inserted this sample edit history entry.
 *      Please log any further modifications made to this file:
 * Steve Cole (SEP), November 20 1992
 *      added scrollbar object
 * Steve Cole (SEP), May 21 1993
 *      Added slider_sync, allowing slider and scrollbar min and
 *      max values to be settable.
 * Glenn Golden (AT&T Bell Labs), Nov. 14 1995
 * 	Fixed bug in slider position evaluation.
 * Dave Nichols (Geco-Prakla), Jan 10 1996
 *      Cleaned up a few memory leaks.

 */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <X11/Xaw/Box.h>
#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Scrollbar.h>

#include <stdio.h>
#include <assert.h>

#include "object.h" 
#include "tree.h"
#include "builders.h"


typedef struct sliderinfo {
    float minval;
    float maxval;
    Widget scrollbar;
    Widget button;
    Widget label;
    char* format;
    char* type;
} _slinfo;


extern void slider_button_callback();
extern void slider_jump_callback();
extern void slider_scroll_callback();
extern void slider_update();
extern void slider_sync();
extern void slider_tab();
extern void slider_destroy();

void build_slider(root,parent,type)
     entry *root;
     Objdef* parent;
     char *type;
{
    Objdef *object;
    char* label;
    struct sliderinfo *slider_info;
    Arg args[20];
    int narg;
    Widget box, labl, scrollbar, vlabel, button, parentw;
    float val,top;
    char text[100];
    int width,height;
    char *orient,*tmp,*tmp2;
    char defname[15];
    static int numslid=1;
    static int numscroll=1;

    parentw = parent->widgetname;
    
    /* slider gets its own box */
    if (!strcmp(type,"slider")) {
      narg = 0;
      common_tags(parentw,root,args,&narg,SET_POSITION);
      box = XtCreateManagedWidget("sliderbox",boxWidgetClass,
                 parentw,args,narg);
    }
    else {
      box = parentw;
    }
    
    /* create new object */
    object = new_object(parent);

    /* construct default slider name */
    if (!strcmp(type,"slider")) {
       sprintf(defname,"slider%d",numslid++);
    } else {
       sprintf(defname,"scrollbar%d",numscroll++);
    }
    
    /* find label, name, action in tree */
    object->name = get_value(root,"name",defname);
    label = get_value(root,"label",object->name);
    object->action = parse_actions(object->name,root);
    slider_info = (struct sliderinfo*) 
                   XtMalloc( sizeof( struct sliderinfo ) );
    object->info = slider_info;
    slider_info->minval=((float)atof(tmp=get_value(root,"min","0")));XtFree(tmp);
    slider_info->maxval=((float)atof(tmp=get_value(root,"max","1")));XtFree(tmp);
    slider_info->label = (Widget)0;
    /* slider value defaults to minimum */
    val = (float) atof(tmp=get_value(root,"value",tmp2=get_value(root,"min","0")));XtFree(tmp);XtFree(tmp2);
    slider_info->format = get_value(root,"format","%f");
    /* save type (scrollbar, slider) for use by callback routines */
    slider_info->type = XtNewString(type);
    
    /* the slider label */
    /* for type scrollbar, no label is displayed */
    if (!strcmp(type,"slider")) {
	narg = 0;
	XtSetArg(args[narg], XtNlabel, label); narg++;
	XtSetArg(args[narg], XtNborderWidth, 0); narg++;
	labl = XtCreateManagedWidget(object->name,labelWidgetClass,box,
				     args,narg);
    }
    XtFree(label);
    
    /* determine the correct starting point for the slider */
    narg = 0;

    top = (val-slider_info->minval)/(slider_info->maxval-slider_info->minval);

    if (sizeof(float) > sizeof(XtArgVal))
      {
	  XtSetArg(args[narg], XtNtopOfThumb, top); narg++;
      }
    else
      {
	  XtArgVal * l_top = (XtArgVal *) &top;
	  XtSetArg(args[narg], XtNtopOfThumb, *l_top); narg++;
      }

    /* common parameters */
    common_tags(box,root,args,&narg,SET_BEFORE & ~SET_SIZE & ~SET_ORIENT);

    /* height, width, orient */
    /* done here instead of common_tags so we could have defaults */
    height = (int) atoi(tmp=get_value(root,"height","25"));XtFree(tmp);
    width = (int) atoi(tmp=get_value(root,"width","100"));XtFree(tmp);
    orient = get_value(root,"orientation","horizontal");
    XtSetArg(args[narg], XtNlength, width); narg++;
    XtSetArg(args[narg], XtNthickness, height); narg++;
    /* solid color thumb if foreground color requested */
    if (is_specified(root,"foreground")) {
       XtSetArg(args[narg], XtNthumb, None); narg++;
       }
    /* the SetTag routine in builders.c does the necessary conversion */
    SetTag(box,args,&narg,"orientation",orient);

    /* slider is actually an athena scrollbar widget */
    assert( narg < 20 );
    scrollbar = XtCreateManagedWidget(object->name,scrollbarWidgetClass,
                      box,args,narg);
    
    /* jump callback is for middle mouse button */
    XtAddCallback( scrollbar, XtNjumpProc, slider_jump_callback, 
		  (XtPointer) object );
    
    /* scroll callback is for incremental scrolling with left and 
       right buttons */
    XtAddCallback( scrollbar, XtNscrollProc, slider_scroll_callback, 
		  (XtPointer) object );
    
    /* set to the correct starting point */
    (void) XawScrollbarSetThumb(scrollbar,
    (val-slider_info->minval)/
             (slider_info->maxval-slider_info->minval),-1.);
    
    /* add a button to print out slider value */
    if (!strcmp(type,"slider") && !all_string_actions(object->action)) {
	button = XtCreateManagedWidget(object->name,commandWidgetClass,box,
				       NULL,ZERO);
	XtAddCallback(button, XtNcallback, slider_button_callback, 
		      (XtPointer) object );
        slider_info->button = button;
    }
    
    sprintf( text, slider_info->format, (float) val);
    object->value = XtNewString(text);
    if (!strcmp(slider_info->type,"slider")) {
       object->widgetname = box;
       object->childwidget = scrollbar;
    } else {
       object->widgetname = scrollbar;
    }

    slider_info->scrollbar = scrollbar;
    object->updater = slider_update;
    object->sync = slider_sync;
    object->tab_proc = slider_tab;
    object->destroy = slider_destroy;
    
    /* now hide object if needed */
    narg = 0;
    common_tags(object->widgetname,root,args,&narg,SET_AFTER);

    /* indicate the slider value using another label */
    if (!strcmp(type,"slider")) {
	narg = 0;
	XtSetArg( args[narg], XtNlabel, object->value); narg++;
	vlabel = XtCreateManagedWidget(object->name,labelWidgetClass,box,
				       args,narg);
	slider_info->label = vlabel;
    }

}

void
  slider_jump_callback(widget, client_data, top_ptr)
Widget widget;
XtPointer client_data, top_ptr;
{
    Objdef *object;
    struct sliderinfo *slider_info;
    Arg arg[1];
    char text[20];
    /* top is new location of slider */
    float top = *((float *) top_ptr);
    object = (Objdef *) client_data;
    slider_info = (struct sliderinfo *) object->info;
    /* compute new value */
    sprintf( text, slider_info->format, 
	    (float) slider_info->minval + 
	    top*(slider_info->maxval - slider_info->minval));
    object->value = XtNewString(text);
    /* set the label to the new value */
    if (slider_info->label != (Widget) 0) {
	XtSetArg( arg[0], XtNlabel, text );
	XtSetValues( slider_info->label, arg, ONE );
    }
    if (!strcmp(slider_info->type,"scrollbar")) 
       perform_actions(object->name,object->action,1);
}

void
  slider_scroll_callback(widget, client_data, pos_ptr)
Widget widget;
XtPointer client_data, pos_ptr;
{
    Objdef *object;
    struct sliderinfo *slider_info;
    Arg arg[1];
    char text[20];
    float top;
    int pos;
    pos = (int) pos_ptr;
    object = (Objdef *) client_data;
    slider_info = (struct sliderinfo *) object->info;
    /* get the current position of the slider */
    XtSetArg( arg[0], XtNtopOfThumb, &top );
    XtGetValues( slider_info->scrollbar, arg, ONE );
    /* now compute new position - 5% change */
    top -= (pos >= 0) ? 0.05 : -0.05;
    if (top > 1.) top = 1.;
    if (top < 0.) top = 0.;
    /* compute new value */
    sprintf( text, slider_info->format, 
	    (float) slider_info->minval +
	    top*(slider_info->maxval - slider_info->minval));
    object->value = XtNewString(text);
    /* update label */
    if (slider_info->label != (Widget) 0) {
	XtSetArg( arg[0], XtNlabel, text );
	XtSetValues( slider_info->label, arg, ONE );
    }
    /* update the slider */
    if (sizeof(float) > sizeof(XtArgVal))
      {
	  XtSetArg(arg[0], XtNtopOfThumb, top);
      }
    else
      {
	  XtArgVal * l_top = (XtArgVal *) &top;
	  XtSetArg(arg[0], XtNtopOfThumb, *l_top);
      }
    XtSetValues( slider_info->scrollbar, arg, ONE );
    if (!strcmp(slider_info->type,"scrollbar")) 
       perform_actions(object->name,object->action,1);
}

/* called when ok button of slider is pressed */
void
  slider_button_callback(widget, client_data, callData)
Widget widget;
XtPointer client_data, callData;
{
    Objdef *object;
    
    object = (Objdef *) client_data;
    perform_actions(object->name,object->action,1);
}

void
  slider_update(object, value)
Objdef *object;
char *value;
{
    struct sliderinfo *slider_info;
    Arg arg[1];
    char text[20];
    float top,val;
    Dimension len;
    int pos;

    slider_info = (struct sliderinfo *) object->info;
    sscanf( value,"%f",&val);
    top = (val - slider_info->minval)/
              (slider_info->maxval-slider_info->minval);
    if (top > 1.) top = 1.;
    if (top < 0.) top = 0.;
    /* compute new value */
    sprintf( text, slider_info->format, 
	    (float) slider_info->minval +
	    top*(slider_info->maxval - slider_info->minval));
    object->value = XtNewString(text);
    /* update label */
    if (slider_info->label != (Widget) 0) {
	XtSetArg( arg[0], XtNlabel, text );
	XtSetValues( slider_info->label, arg, ONE );
    }
    /* update the slider */
    if (sizeof(float) > sizeof(XtArgVal))
      {
	  XtSetArg(arg[0], XtNtopOfThumb, top);
      }
    else
      {
	  XtArgVal * l_top = (XtArgVal *) &top;
	  XtSetArg(arg[0], XtNtopOfThumb, *l_top);
      }
    XtSetValues( slider_info->scrollbar, arg, ONE );
}

/*
 * this routine is called when a slider or scrollbar has
 * a resource changed with SET. Setting of min and max is
 * handled here, because we must take care to update the
 * position of the scrollbar. 
 */
void slider_sync(object,tag,value)
Objdef* object;
char* tag;
char* value;
{
    struct sliderinfo *slider_info;

    slider_info = (struct sliderinfo *) object->info;

    /* handle most tags with update_common_tag */
    update_common_tag(object,tag,value);

    /* handle min and max here */
    if (!strcmp(tag,"min"))
      slider_info->minval = ((float) atof(value));
    if (!strcmp(tag,"max"))
      slider_info->maxval = ((float) atof(value));

    /* call slider update to position scrollbar given new limits */
    slider_update(object,object->value);

}

/*
 * This routine is called to tab to this object
 * We try and warp to the thumb position.
 */
void slider_tab(object)
Objdef* object;
{
  struct sliderinfo *slider_info;
  Widget w;

  slider_info = (struct sliderinfo *) object->info;
  w = slider_info->scrollbar;
  XWarpPointer(XtDisplay(w),None,XtWindow(w), 0,0,0,0,5,5);

}

/*
 * This routine is called to free up dynamically allocated memory
 * in the info structure
 */
void slider_destroy( object )
Objdef* object;
{
  struct sliderinfo *slider_info;
  slider_info = (struct sliderinfo *) object->info;
  XtFree( slider_info -> type );
  XtFree( slider_info -> format );
}
