/*
 * Copyright 1992, 1993, 1994 The Board of Trustees of the Leland Stanford
 * Junior University. Copyright 1994, 1995 Dave Nichols and Steve Cole.
 * Permission is hereby given to use, copy, modify, and
 * distribute this software, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * STANFORD AND THE AUTHORS MAKE NO REPRESENTATIONS OR WARRANTIES OF ANY 
 * KIND CONCERNING  THIS SOFTWARE.
 */

/*
 *
 *  source file:   ./xtpanel/template.c
 *
 * Dave Nichols (SEP), July 17 1993
 *      wrote template.c
 */

#include <stdio.h>

#include "object.h"
#include "tree.h"
#include "builders.h"
#include "string_buf.h"

struct tmpl_info {
	char* name;
	entry* root;
	struct tmpl_info* next;
};

static struct tmpl_info *tmpl_head = ( struct tmpl_info*)0;

void new_template( name )
char * name;
{

struct tmpl_info *curr, *newtmpl;

/* find the tail, and check for duplicate names */
if( tmpl_head != 0 ){
for( curr = tmpl_head; curr->next != 0 ; curr=curr->next ){
    if( !strcmp( curr->name, name ) ) {
	fprintf(stderr," Attempt to define two templates called %s\n",name);
	exit(-1);
    }
}
}

/* make a new one and add it to the tail */
newtmpl = (struct tmpl_info *) XtMalloc( sizeof(struct tmpl_info) );
newtmpl->name = XtNewString(name);
newtmpl->next = 0;
newtmpl->root = 0;

if( tmpl_head == 0 ){
    tmpl_head = newtmpl;
}else{
    curr->next = newtmpl;
}

}


/* find the template with the matching name */
static struct tmpl_info* get_template( name )
char* name;
{
   struct tmpl_info *curr ;

   for( curr = tmpl_head; curr != 0 ; curr=curr->next ){
    if( !strcmp( curr->name, name ) ) return curr;
   }

   return 0;
}


void save_template( root )
    entry *root;
{

char* tmplname;
struct tmpl_info *newtmpl;

tmplname = get_value( root, "name", "NONAME" );

if( !strcmp( tmplname, "NONAME" ) ){
	fprintf( stderr, "A template must have a name\n");
	exit(-1);
}

newtmpl = get_template( tmplname);

if( newtmpl == 0 ){
    fprintf( stderr, "This template has not been defined: %s \n", tmplname);
    exit(-1);
}

newtmpl->root = new_entry( "template",8); 
newtmpl->root->child = root->child;

}


int is_template( name )
char* name;
{
return ( get_template( name ) != 0 );
}


/*
 * Function name: one_word
 * Description: extract one word from the line and return a pointer to the
 * position after the word. The word is returned in char** word.
 * A word is defined by separating space or tab.
 * (like strtok, but we don't want the line modified).
 */
static char* one_word( line, word )
char* line;
char** word;
{
int len;

/* find the end of the word */
len = strcspn( line, " \t" ); 

/* allocate and copy */
*word = (char*)XtMalloc( len + 1); 
strncpy( *word, line, len ); 
(*word)[len]='\0'; 

/* return pointer to space after word */
return(line+len);
}

/* for every string in the tree, look for oldname
 * If it is followed by a word then look up that value in inst_tree
 * If an entry exists replace oldname:word by the entry value
 * If it doesn't exist replace oldname by newname */
static void massage_tree( tree, oldname, newname, inst_tree )
entry* tree;
char* oldname;
char* newname;
entry* inst_tree;
{

char* mark;
char* curr;
char* end;
char* word;
char* new_val;
string_buf* new_buf;

  if( tree->value != 0 ){

    new_buf = buf_start();

    curr = tree->value;
    end  = tree->value + strlen( tree->value );

    while( ( mark = strstr( curr, oldname )) != NULL ){
      buf_cat( new_buf, curr, mark-curr ); /* copy up to the mark */
      curr = mark+strlen(oldname);        /* move past oldname */
      curr = one_word( curr, &word );      /* look for the following word */
      /* look it up in inst_tree */
      new_val = get_raw_value(inst_tree,word,"xtp_not_found");
     
      if( ! strcmp( new_val, "xtp_not_found" ) ){ 
          buf_cat( new_buf, newname, strlen(newname ) ); /* add newname */
          buf_cat( new_buf, word, strlen(word ) ); /* add word */
      }else{
	  buf_cat(new_buf, new_val, strlen(new_val)); /*add subsituted value*/
      }
      XtFree( new_val); XtFree( word );
    }

    if( end-curr > 0 ){
	buf_cat( new_buf, curr, end-curr );
    }
    XtFree( tree->value );
    tree->value = buf_fetch( new_buf );
    buf_free( new_buf );

  }

  if( tree->next != 0 ) {
        massage_tree( tree->next, oldname, newname, inst_tree );
  }
  if( tree->child != 0 ) {
        massage_tree( tree->child, oldname, newname, inst_tree  );
  }

}

void build_template(root,parent,name)
     entry *root;
     Objdef* parent;
     char* name;
{

struct tmpl_info* template;
entry* newtree;
entry *curr,*prev;
char* newname;
char *name1, *name2;


    template = get_template( name );
    if( template == 0 ){
	fprintf( stderr, "unable to find template named %s\n",name);
	fprintf( stderr, "templates must be defined before they are used\n");
	exit(-1);
    }

    newtree = copy_tree( template->root );
 
    /* find name in tree */
    newname = get_value(root,"name","");

    if ( !strcmp( newname,"") ){
	fprintf(stderr,"When using templates a name must be specified\n");
	exit(-1);
    }

    name1 = (char*)XtMalloc( strlen( newname ) +2 );
    strcpy( name1, newname ); strcat( name1, ":" );

    name2 = (char*)XtMalloc( strlen( name ) +2 );
    strcpy( name2, name ); strcat( name2, ":" );


    /* fix up the object names in the new tree */
    massage_tree( newtree, name2, name1, root );

    /* remove the name entry from the first level of the new tree */
    prev=0;
    for( curr = newtree->child; curr != 0 ; curr = curr->next ){
	if( !strcmp( curr->tag, "name" )){
	    if( prev == 0 ){ newtree->child = curr->next;
	    }else{ prev->next = curr->next;}
	}else{
	  prev=curr;
	}
    }
	
    
    process_tree( newtree, parent );

}
