/*
 * Copyright 1992, 1993, 1994 The Board of Trustees of the Leland Stanford
 * Junior University. Copyright 1994, 1995 Dave Nichols and Steve Cole.
 * Permission is hereby given to use, copy, modify, and
 * distribute this software, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * STANFORD AND THE AUTHORS MAKE NO REPRESENTATIONS OR WARRANTIES OF ANY 
 * KIND CONCERNING  THIS SOFTWARE.
 */


/*
 *
 *  source file:   ./xtpanel/text.c
 *
 * Steve Cole, Dave Nichols (SEP), August 28 1992
 *      Inserted this sample edit history entry.
 *      Please log any further modifications made to this file:
 * Steve Cole (SEP), February 23 1993
 *      new object "field" - single line text widget.
 * Steve Cole (SEP), May 6 1993
 *      only string actions are performed when leaving a field.
 * Steve Cole (SEP), June 7 1993
 *      Both text and field objects now perform all actions
 *      when cursor leaves the text field. This seems to be
 *      the best way.
 * Steve Cole (SEP), September 23 1993
 *	Changed the default sizing. Text objects by default are
 *	big enough to contain the initial text, and add scrollbars
 *      if the text gets larger as needed. Field objects also contain
 *      the text, and grow horizontally as needed. Both have certain
 *      minimum sizes, and the user can always hardwire height and width.
 * Steve Cole (SEP), June 2 1994
 *	Made an ifdef to control what actions take place when the
 *	cursor leaves a field object. The default is that only
 *	string type actions - NONE and STRING - occur.
 *	FIELDEXIT is defined (see Imake.config) then all actions occur.
 *      In either case, all actions occure when the return key is
 *      pressed in the field.
 */

#include <stdio.h> 

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Text.h>
#include <assert.h>

#include "object.h"
#include "tree.h"
#include "builders.h"

/* translation table used for text object */
char text_trans[] =
  "<Leave>:         update_text()";

/* translation table used for field object */
char field_trans[] =
  "<Leave>:         update_field()\n\
   <Key>Return:     field_return_key()"; 

void build_text(root,parent,type)
     entry *root;
     Objdef* parent;
     char *type;
{
    Objdef *object;
    Arg args[20];
    int narg;
    Widget text;
    char defname[12];
    int width,height;
    static int numtext=1, numfield=1;
    char *buff,*line,*tmp;
    int nline,len,maxlen;
    Position topMargin, bottomMargin, leftMargin, rightMargin;
    XFontStruct *font;
    extern void text_update();
    
    /* create new object */
    object = new_object(parent);

    /* construct default text widget name */
    if (!strcmp(type,"text"))
       sprintf(defname,"text%d",numtext++);
    else
       sprintf(defname,"field%d",numfield++);
    
    /* find value, name, etc. in tree */
    object->name = get_value(root,"name",defname);
    object->action = parse_actions(object->name,root);
    object->value = get_value(root,"value","");
    
    narg=0;

    /* set the text */
    narg = 0;
    XtSetArg(args[narg], XtNstring, object->value); narg++;

    /* text object adds scrollbars if needed, and height can be set */
    if (!strcmp(type,"text"))
    {
        if (is_specified(root,"width") ){
           XtSetArg(args[narg], XtNscrollHorizontal, XawtextScrollWhenNeeded); 
           narg++;
	   if( is_specified(root,"height")) {
              XtSetArg(args[narg], XtNscrollVertical, XawtextScrollWhenNeeded); 
              narg++;
	   }else{
              XtSetArg(args[narg], XtNresize, XawtextResizeHeight); narg++;
	   }
        }else{
	   if( is_specified(root,"height")) {
              XtSetArg(args[narg], XtNscrollVertical, XawtextScrollWhenNeeded); 
              narg++;
              XtSetArg(args[narg], XtNresize, XawtextResizeWidth); narg++;
	   }else{
              XtSetArg(args[narg], XtNresize, XawtextResizeBoth); narg++;
	   }
        }

	if (is_specified(root,"height")) {
    		height=(int)atoi(tmp=get_value(root,"height","0"));XtFree(tmp);
    		XtSetArg(args[narg], XtNheight, height); narg++;
	}

    /* field object grows horizontally as needed, and is always read-write */
    } else {
        XtSetArg(args[narg], XtNresize, XawtextResizeWidth); narg++;
	XtSetArg(args[narg], XtNeditType, XawtextEdit); narg++;
    }

    /* width can be specified for both */
    if (is_specified(root,"width")) {
      	width = (int) atoi(tmp=get_value(root,"width","0"));XtFree(tmp);
   	XtSetArg(args[narg], XtNwidth, width); narg++;
    }

    /* common parameters */
    common_tags(parent->widgetname,root,args,&narg,SET_BEFORE & ~SET_SIZE);

    assert( narg < 20 );
    text = XtCreateManagedWidget(object->name,asciiTextWidgetClass,
               parent->widgetname,args,narg);
    
    /* if height or width not user-specified, we need to compute it */
    if (!is_specified(root,"height") || !is_specified(root,"width")) {

      /* find font and margin info */
      XtVaGetValues(text,
            XtNfont, &font,
            XtNtopMargin, &topMargin,
            XtNbottomMargin, &bottomMargin,
            XtNleftMargin, &leftMargin,
            XtNrightMargin, &rightMargin,
            NULL
      );

      /* compute number of lines and width of each in pixels */
      maxlen = 0;
      /* must use a copy to avoid destroying the value */
      buff = XtNewString(object->value);
      for (line = strtok(buff,"\n"), nline=0;
           line != (char*)0;
           line = strtok( (char*)0,"\n"), nline++) {
        len = XTextWidth(font, line, strlen(line));
	if (len > maxlen) maxlen = len;
      }
      /* free the storage for the copy */
      XtFree(buff);

      /* enforce minimum values */
      if (!strcmp(type,"text")) { 
	if (maxlen < 300) maxlen = 300;
    	if (nline < 3) nline = 3;
      } else {
	if (maxlen < 100) maxlen = 100;
	nline = 1;
      }

      /* compute height in pixels, add margins to height and width */
      /* might be nice to pad by scrollbar thickness for text object */
      height = nline * (font->ascent + font->descent) 
		+ topMargin + bottomMargin;
      width = maxlen + leftMargin + rightMargin;

      /* set the height and width */
      narg = 0;
      if (!is_specified(root,"width")){
      	XtSetArg(args[narg], XtNwidth,  width); narg++;
      }
      if (!is_specified(root,"height")) {
      	XtSetArg(args[narg], XtNheight, height); narg++;
      }
      if( narg > 0 ) XtSetValues(text,args,narg);

    }

    object->widgetname = text;
    object->updater = text_update;

    /* now hide object if needed */
    narg = 0;
    common_tags(text,root,args,&narg,SET_AFTER);

    /* use translations to update value whenever mouse leaves widget */
    if (!strcmp(type,"text")){
      XtOverrideTranslations(text,XtParseTranslationTable(text_trans));
    }else{
      XtOverrideTranslations(text,XtParseTranslationTable(field_trans));
    }

}

/*
 * this routine gets called whenever we leave
 * a text widget, to be sure its value is up to date
 */
void text_update_callback( w, event, params, num_params)
     Widget w;
     XEvent *event;
     String *params;
     Cardinal *num_params;
{
    Objdef* object;
    char *value;
    Arg args[20];
    int narg;

    object = find_by_widget(w);
    narg = 0;
    XtSetArg(args[narg], XtNstring, &value); narg++;
    XtGetValues(object->widgetname,args,narg);
    if( object->value != 0 ) XtFree(object->value);
    object->value = XtNewString(value);
    perform_actions(object->name,object->action,1);
}

/*
 * this routine gets called whenever we leave
 * a field object
 */
void field_update_callback( w, event, params, num_params)
     Widget w;
     XEvent *event;
     String *params;
     Cardinal *num_params;
{
    Objdef* object;
    char *value;
    Arg args[20];
    int narg;

    object = find_by_widget(w);
    narg = 0;
    XtSetArg(args[narg], XtNstring, &value); narg++;
    XtGetValues(object->widgetname,args,narg);
    if( object->value != 0 ) XtFree(object->value);
    object->value = XtNewString(value);
#ifdef FIELDEXIT
    perform_actions(object->name,object->action,1);
#else
    perform_actions(object->name,object->action,0);
#endif
}

/*
 * this routine gets called whenever we press return
 * in a field object
 */
void field_return_key_callback( w, event, params, num_params)
     Widget w;
     XEvent *event;
     String *params;
     Cardinal *num_params;
{
    Objdef *object;
    char *value;
    Arg args[20];
    int narg;

    object = find_by_widget(w);
    narg = 0;
    XtSetArg(args[narg], XtNstring, &value); narg++;
    XtGetValues(object->widgetname,args,narg);
    if( object->value != 0 ) XtFree(object->value);
    object->value = XtNewString(value);
    perform_actions(object->name,object->action,1);
    return;
}

/* this gets called by ASSIGN actions */
void text_update(object,value)
Objdef* object;
char* value;
{
    Arg args[20];
    int narg;
    char *thestring;
    int len, ret;
    XawTextPosition start,end;
    XawTextEditType etype;
    XawTextBlock *text;

    /* determine the editType */
    narg = 0;
    XtSetArg(args[narg], XtNstring, &thestring); narg++;
    XtSetArg(args[narg], XtNeditType, &etype); narg++;
    XtGetValues(object->widgetname,args,narg);
    len = strlen(thestring);
 
    /* if append, find the end of the text and add on */
    if (etype == XawtextAppend) {
	start = (long) len+1;
	end = start;
        text = (XawTextBlock *) XtMalloc( sizeof(XawTextBlock));
	/* add a carriage return first */
	text->ptr = value;
	text->length = strlen(text->ptr);
	text->firstPos = 0;
	text->format = FMT8BIT;
        ret = XawTextReplace(object->widgetname,start,end,text);
	if (ret != 0) 
            fprintf(stderr,"XawTextReplace return code is %d\n",ret);
	if (object->value != 0) XtFree(object->value);
	object->value = (char*)XtMalloc( len + text->length + 1);
        strcpy(object->value,thestring);
        strcat(object->value,value);
	XawTextSetInsertionPoint(object->widgetname,len+text->length);
	XtFree((char*)text);

    /* otherwise replace the whole thing */
    } else {
      narg = 0;
      XtSetArg(args[narg], XtNstring, value); narg++;
      XtSetValues(object->widgetname,args,narg);
      if (object->value != 0) XtFree(object->value);
      object->value = XtNewString(value);
    }
    perform_actions(object->name,object->action,0);
}
