/*
 * Copyright 1992, 1993, 1994 The Board of Trustees of the Leland Stanford
 * Junior University. Copyright 1994, 1995 Dave Nichols and Steve Cole.
 * Permission is hereby given to use, copy, modify, and
 * distribute this software, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * STANFORD AND THE AUTHORS MAKE NO REPRESENTATIONS OR WARRANTIES OF ANY 
 * KIND CONCERNING  THIS SOFTWARE.
 */


/*
 *
 *  source file:   ./xtpanel/timer.c
 *
 * Steve Cole, Dave Nichols (SEP), August 28 1992
 *      Inserted this sample edit history entry.
 *      Please log any further modifications made to this file:
 * Dave Nichols, Steve Cole (SEP), May 5 1993
 *	wrote timer.c.
 * Steve Cole (SEP), June 7 1993
 *      True/False can be used when setting "running" attribute.
 * Dave Nichols (SEP), July 6 1993
 *      Fix double timer starts when a timer sets in its own action
 *      Fix suggested by  Tony Stoneley <ajms@cus.cam.ac.uk>.
 * Dave Nichols (Geco-Prakla), Jan 30 1995
 *      Fix tiny memory leak.
 */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>


#include <stdio.h>
#include <math.h>

#include "object.h" 
#include "tree.h"
#include "builders.h"

typedef struct timerinfo {
    unsigned long interval;
    Boolean running;
    XtIntervalId id;
    int in_callback;
} _tinf;

extern XtAppContext app_con;
extern void timer_destroy();
extern void timer_sync();

void build_timer(root,parent)
     entry *root;
     Objdef *parent;
{
    Objdef *object;
    struct timerinfo *timer_info;
    char defname[12];
    Widget timer;
    static int numtim=1;
    extern void timer_callback();
    char *runflg,*tmp;
    
    /* create new object */
    object = new_object(parent);

    timer_info = (struct timerinfo*) XtMalloc( sizeof(struct timerinfo) );

    /* construct default timer name */
    sprintf(defname,"timer%d",numtim++);
    
    /* find label, name, action in tree */
    object->name = get_value(root,"name",defname);
    object->action = parse_actions(object->name,root);
    object->info = timer_info;
    object->destroy = timer_destroy;
    object->sync = timer_sync;
    timer_info->interval = 
      (unsigned long)(atof(tmp=get_value(root,"interval","60"))*1000.);XtFree(tmp);
    runflg = get_value(root,"running","True");
    timer_info->running = get_boolean(parent->widgetname,runflg);
    XtFree( runflg );
    timer_info->in_callback = 0;

    if( timer_info->running ){
         timer_info->id = XtAppAddTimeOut(app_con, timer_info->interval, 
		timer_callback, (XtPointer)object);
    }
}

/* callback used for timers */
void timer_callback( client_data, id)
XtPointer client_data ;
XtIntervalId* id;
{
    Objdef *object;
    Arg arg[1];
    struct timerinfo *timer_info;
    
    object = (Objdef *) client_data;
    timer_info = (struct timerinfo*)object->info;


    if( *id != timer_info->id ){
	fprintf(stderr,"Timer callback error, ids do not match %d %d\n",
		*id, timer_info->id );
    }

    /* protect against the SET routine restarting the timer */
    timer_info->in_callback = 1;

    perform_actions(object->name,object->action,1);

    timer_info->in_callback = 0;

    if( timer_info->running ){
         /* re-register the timeout */
         timer_info->id = XtAppAddTimeOut(app_con, timer_info->interval, 
		timer_callback, (XtPointer)object);
    }

}

/*
 * this routine is called when a timer's state (running
 * or not) or interval is changed by the SET action.
 */
void timer_sync(object,tag,value)
Objdef* object;
char* tag;
char* value;
{
    struct timerinfo *timer_info;

    timer_info = (struct timerinfo *) object->info;
    
    /* remove old timeout so we can start over */
    if( timer_info->running ){
       XtRemoveTimeOut(timer_info->id);
    }

    if (!strcmp(tag,"running")) {
       timer_info->running = get_boolean(object->parent->widgetname,value);
    }

    if (!strcmp(tag,"interval")) {
       timer_info->interval = (unsigned long) (atof(value)*1000.);
    }

    /* set it going again unless we were called from the callback
     * In that case the callback will start it again */
    if( timer_info->running && !timer_info->in_callback ){
         timer_info->id = XtAppAddTimeOut(app_con, timer_info->interval, 
		timer_callback, (XtPointer)object);
    } 
}

/*
 * this routine is called to destroy a timer
 */
void timer_destroy(object)
Objdef* object;
{
    struct timerinfo *timer_info;

    timer_info = (struct timerinfo *) object->info;
    /* remove the timeout */
    XtRemoveTimeOut(timer_info->id);
}
