/*
 * Copyright 1992, 1993, 1994 The Board of Trustees of the Leland Stanford
 * Junior University. Copyright 1994, 1995 Dave Nichols and Steve Cole.
 * Permission is hereby given to use, copy, modify, and
 * distribute this software, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * STANFORD AND THE AUTHORS MAKE NO REPRESENTATIONS OR WARRANTIES OF ANY 
 * KIND CONCERNING  THIS SOFTWARE.
 */


/*
 *
 *  source file:   ./xtpanel/window.c
 *
 * Steve Cole, Dave Nichols (SEP), August 28 1992
 *      Inserted this sample edit history entry.
 *      Please log any further modifications made to this file:
 * Steve Cole (SEP), June 7 1993
 *      wrote window.c
 * Dave Nichols (Geco-Prakla), Jan 30 1995
 *      Fixed tiny memory leak (to make purfiy keep quiet)
 * Dave Nichols (Geco-Prakla), Aug 20 1995
 *      Make window name the label not the object name.
 *      Allow attribute change of label on window to change the window label
 */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <X11/Shell.h>
#include <X11/Xaw/Box.h>

#include <stdio.h>
#include <assert.h>


#include "object.h"
#include "tree.h"
#include "builders.h"

#include "wm_protocol.h"

void window_protocols( );

void build_window(root,parent,create)
     entry *root;
     Objdef* parent;
     int create;
{
    Objdef *object;
    char *name,*visible,*label;
    Widget window,box;
    Arg args[20];
    int narg;
    char defname[12];
    static int numwin=1;
    struct compositeinfo *window_info;
    extern void window_destroy();
    extern void window_sync();

    /* create new object */
    object = new_object(parent);

    /* construct default dialog name */
    sprintf(defname,"window%d",numwin++);
    
    /* find name in tree */
    object->name = get_value(root,"name",defname);
    /* now get the label on the window */
    label = get_value( root, "label", object->name );

    window_info = (struct compositeinfo*)
                   XtMalloc( sizeof( struct compositeinfo ) );

    /* create flag indicates whether a new shell window is needed */
    if (create) {
         narg = 0;
	 /* let shell resize itself if needed */
         XtSetArg(args[narg], XtNallowShellResize, TRUE ); narg++;
         window = XtCreatePopupShell(label,topLevelShellWidgetClass,
				parent->widgetname,args,narg);
    } else {
	 window = parent->widgetname;
    }
    /* common parameters */
    narg = 0;
    common_tags(parent->widgetname,root,args,&narg,SET_BEFORE);

    assert ( narg < 20 );
    box = XtCreateManagedWidget(object->name,boxWidgetClass,
				window,args,narg);

    object->widgetname = box; 

    window_info->topwidget = window;
    object->info = window_info;
    object->destroy = window_destroy;
    object->sync = window_sync;

    /* now hide object if needed */
    narg = 0;
    common_tags(window,root,args,&narg,SET_AFTER);

    /* now process the tree to build all the objects rooted in this window */
    process_tree( root, object );

    /* pop it up unless visible=false */
    visible = get_value( root, "visible", "True" );
    if( create && get_boolean(window,visible) )  XtPopup(window,XtGrabNone);	    XtFree( visible );

    XtFree( label );

    return;
}

void window_protocols( object )
Objdef* object;
{
    struct compositeinfo *window_info;
    Widget window;

    window_info = (struct compositeinfo*) object->info;

    window = window_info->topwidget; 
    /* What to do when we get the close message  we choose to destroy */
    handle_WMPROTOCOL(window,WM_CLOSE_DESTROY);
/*    handle_WMPROTOCOL(window,WM_CLOSE_EXIT);*/
/*    handle_WMPROTOCOL(window,WM_CLOSE_POPDOWN);*/
}
 
void window_sync(object,tag,value)
Objdef* object;
char* tag;
char* value;
{

    struct compositeinfo *window_info;
    window_info = (struct compositeinfo*) object->info;

    /*
     * visible must be handled here because we need to
     * call XtPopup and XtPopdown, not the manage routines
     * that are called for most objects.
     */
    if( !strcmp(tag,"visible"))
    {
	if (get_boolean(object->widgetname,value)){
        	XtPopup(XtParent(object->widgetname), XtGrabNone);
	}else{
        	XtPopdown(XtParent(object->widgetname));
	}
    } else if( !strcmp( tag, "label" ) ){
	if( XtIsRealized(window_info->topwidget) ){
	    XStoreName( XtDisplay(window_info->topwidget),
                    XtWindow(window_info->topwidget), value );
	}
    } else {
    /* send everything else to update_common_tag */
        update_common_tag(object,tag,value);
    }
}

/* this routine destroys a window */
void window_destroy(object)
Objdef* object;
{
    struct compositeinfo *window_info;

    window_info = (struct compositeinfo*) object->info;

    /* destroy the outermost widget */
    XtDestroyWidget(window_info->topwidget);
}
