/*
 * Copyright 1992, 1993, 1994 The Board of Trustees of the Leland Stanford
 * Junior University. Copyright 1994, 1995 Dave Nichols and Steve Cole.
 * Permission is hereby given to use, copy, modify, and
 * distribute this software, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * STANFORD AND THE AUTHORS MAKE NO REPRESENTATIONS OR WARRANTIES OF ANY 
 * KIND CONCERNING  THIS SOFTWARE.
 */


/*
 * wm_protocol.c -- MWM close button handler.
 *
 * Based on code posted to the internet by  
 * williams@herky.cs.uiowa.edu (Kent Williams)
 *
 * Modified 15 Dec 1993 : Dave Nichols (SEP) 
 *    Modified to fit into xtpanel.
 */
#include <stdio.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xatom.h>
#define WM_PROTOCOL_DEFS 1
#include "wm_protocol.h"

#if defined __STDC__ || defined __stdc__ 
void close_window(Widget widget,
  XEvent *event,
  String *params,
  Cardinal *num_params)
#else
void close_window( widget, event, params, num_params)
Widget widget;
XEvent *event;
String *params;
Cardinal *num_params;
#endif
{
  if(*num_params != 1)
    abort();
  if(!strcmp(params[0],WM_CLOSE_EXIT)) {
    quit_xtpanel( 0 );
  } else if(!strcmp(params[0],WM_CLOSE_POPDOWN)) {
    XtPopdown(widget); /* just pop this widget down */
  } else if(!strcmp(params[0],WM_CLOSE_DESTROY)) {
    XtPopdown(widget); /* we want this widget dead. */
    quit_xtpanel( 0 );
  } else if(!strcmp(params[0],WM_CLOSE_WARN)) {
    /* you could probably come up with something more useful and polite
     * to do here.
     */
    fprintf(stderr,"Don't do that, ya moron!\n");
    return;
  }
}

/*
 * AFTER you've realized and popped up a widget,
 * call this function to register an action for it.
 * Note that as a side effect of calling this with an
 * arbitrary string not in the set given above, the close
 * button will just be ignored.
 */
#if defined __STDC__ || defined __stdc__ 
void handle_WMPROTOCOL(Widget w,char *acttype)
#else
void handle_WMPROTOCOL(w,acttype)
Widget w;
char *acttype;
#endif
{
  static int init = 0;
  static Atom wm_delete_window;
  static XtActionsRec actionTable[] = {
    { "WMclose_window", close_window },
  };
  char transbuf[256];

  if(!init) {
    XtAppAddActions(XtWidgetToApplicationContext(w),
                    actionTable, XtNumber(actionTable));
    wm_delete_window = XInternAtom(XtDisplay(w),"WM_DELETE_WINDOW",False);
    init++;
  }

  XSetWMProtocols(XtDisplay(w),XtWindow(w),&wm_delete_window,1);
  sprintf(transbuf,
          "<Message>WM_PROTOCOLS: WMclose_window(%s)\n",acttype);
  XtOverrideTranslations(w,XtParseTranslationTable(transbuf));
}

