/*
 * Copyright 1992, 1993, 1994 The Board of Trustees of the Leland Stanford
 * Junior University. Copyright 1994, 1995 Dave Nichols and Steve Cole.
 * Permission is hereby given to use, copy, modify, and
 * distribute this software, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * STANFORD AND THE AUTHORS MAKE NO REPRESENTATIONS OR WARRANTIES OF ANY 
 * KIND CONCERNING  THIS SOFTWARE.
 */


/*
 *
 *  source file:   ./xtpanel/xtpanel_init.c
 *
 * Dave Nichols (SEP), Feb 8 1993
 *      Inserted this sample edit history entry.
 *      Please log any further modifications made to this file:
 *
 * Glenn Golden, (AT&T Bell Labs), July 3, 1995
 *	Added sciplot object to the Big Switch.
 *
 * Dave Nichols (Geco-Prakla), Dec 8 1995
 *	Added beNiceToColormap (true) as default resource for XAW3D
 */


#include <stdio.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <X11/Xaw/Box.h>
#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Viewport.h>
#include <X11/Xmu/Converters.h>

#include <X11/Xos.h>

#include "tree.h"
#include "object.h"
#include "builders.h"

String fallback_resources[] = {
    "*input:                  			True",
    "*Paned*resizable:   			True",
    "*Dialog*Command.label:   			ok",
    "*sliderbox*Command.label:			ok",
    "*Dialog*resizable:       			True",
    "*Dialog*value.translations: #override <Key>Return: return_key()",
    "*Box*allowResize:        			True",
    "*Box*resizable:        			True",
    "*SimpleMenu*leftMargin:	         	30",
    "*noborder.borderWidth:          		0",
    "*squeezebox*vSpace:                        0",
    "*squeezebox*hSpace:                        0",
    "*squeezebox*defaultDistance:               0",
    "*Form*top:                      		ChainTop",
    "*Form*bottom:                  		ChainTop",
    "*Form*left:                     		ChainLeft",
    "*Form*right:                    		ChainLeft",
    "*beNiceToColormap:                    	false",
#ifdef XAW_ARROW_SCROLLBAR
    "*graphbox*Scrollbar.translations:#override\\n\
      Meta<EnterNotify>: MoveThumb() NotifyThumb()\\n\
      Meta<LeaveNotify>: NotifyScroll(Proportional) EndScroll()\\n\
      Meta<Motion>: MoveThumb() NotifyThumb()",
#else
    "*graphbox*Scrollbar.translations:#override\\n\
      Meta<EnterNotify>: StartScroll(Continuous) MoveThumb() NotifyThumb()\\n\
      Meta<LeaveNotify>: NotifyScroll(Proportional) EndScroll()\\n\
      Meta<Motion>: MoveThumb() NotifyThumb()",
#endif
    "*togglebox*Toggle.translations:#override\\n\
      Meta<EnterNotify>:                set()\\n\
      Ctrl<EnterNotify>:                unset()",
    NULL,
};

extern void return_key_callback(),dialog_update_callback();
extern void text_update_callback(),graph_update_callback();
extern void field_return_key_callback(),grid_update_callback();
extern void field_update_callback(),quit_callback();
extern void tab_next_callback();

static XtActionsRec window_actions[] = {
    {"return_key",       return_key_callback},
    {"field_return_key", field_return_key_callback},
    {"update_dialog",    dialog_update_callback},
    {"update_text",      text_update_callback},
    {"update_field",     field_update_callback},
    {"tab_next",         tab_next_callback},
    {"update_graph",     graph_update_callback},
    {"update_grid",      grid_update_callback},
    {"quit",      	 quit_callback},
};

int		quitflag=0;

Widget toplevel;
XtAppContext app_con;
char* progname;
int stdin_input=0;
int stdin_used=0;

int io_err_handler( disp ) Display * disp; { exit( 0 ); }
extern void quit_xtpanel();
extern void parse_args();
extern void process_tree();

/* Initialize xtpanel assuming that this is the only x-windows part
 * of the code. This routine will open the connection to the display
 * and create the top-level window etc.
 *
 * Pass an argument list as defined by the xtpanel man page
 *
 */

int xtp_init_app(argc,argv)
     int argc;
     char **argv;
{
    entry *root;
    Arg args[20];
    int narg;
    Objdef* rootobj;
    
    /* set the io error handler */
    XSetIOErrorHandler( io_err_handler );

    /* this allows the xtpanel window to grow if needed */
    narg = 0;
    XtSetArg(args[narg], XtNallowShellResize, TRUE ); narg++;

    toplevel = XtAppInitialize(&app_con, "XTpanel", NULL, ZERO,
			       &argc, argv, fallback_resources,
			       args, narg);

    rootobj = new_object(0); 
    rootobj->widgetname = toplevel; 
    rootobj->name = XtNewString("toplevel"); 

    XtAddCallback( toplevel, XtNdestroyCallback, quit_callback, NULL);
    
    XtAppAddActions(app_con, window_actions, XtNumber(window_actions));

    /* make the root entry to the tree, it is called "root" */
    root= new_entry( "root", 4);
    
    /* now go and parse what is left of the command line */
    progname = XtNewString( *argv );
    argv++; argc--;
    
    parse_args( argc, argv, root );
    
    /* parse stdin if it pointing at something unless stdin_input is set */
    if( (!isatty( fileno(stdin) ))  && (!stdin_input) ) {
        parse_file( stdin, root );
	stdin_used=1;
    }
    
    /* print tree for debugging */
    /* print_entry( stderr,root,0 ); */
   
    /* create top level window  and all objects in it */
    build_window(root, rootobj, 0);

    /* clean up the parsed representation */
    delete_entry( root );

    XtRealizeWidget(toplevel);
    XSync( XtDisplay(toplevel), True );

    /*  participate in WM_PROTOCOLS */
    window_protocols( find_by_name("window1") ); 
    
    /* pass control to notifier */
    XtAppMainLoop(app_con);

    return 0;
}

/* initalize xtpanel within an exiting widget provided by the user 
 * Everything will be put in a box inside that widget 
 * Pass an application context, a parent widget and the name of the
 * file to be parsed */

xtp_init_file( user_app_con, parent, filename )
XtAppContext user_app_con;
Widget parent;
char* filename;
{
    entry *root;
    Objdef* rootobj;
   
    toplevel = parent; 
    rootobj = new_object(0); 
    rootobj->widgetname = parent; 
    rootobj->name = XtNewString("toplevel"); 

    XtAppAddActions(app_con, window_actions, XtNumber(window_actions));

    /* make the root entry, it is called "root" */
    root= new_entry( "root", 4);

    /* parse the input file into a tree rooted at this point */
    parse_file_named( filename, root, 0, (char**)0 );

    /* create box within user's widget and put  all objects in it */
    build_box( root, rootobj, 0 );

    /* clean up the parsed representation */
    delete_entry( root );

    return 0;
}

/* initalize xtpanel within an exiting widget provided by the user 
 * Everything will be put in a box inside that widget 
 * Pass an application context, a parent widget and a buffer containing
 * the text to be parsed */

xtp_init_buffer( user_app_con, parent, buffer )
XtAppContext user_app_con;
Widget parent;
char* buffer ;
{
    entry *root;
    Objdef* rootobj;
   
    toplevel = parent; 
    rootobj = new_object(0); 
    rootobj->widgetname = parent; 
    rootobj->name = XtNewString("toplevel"); 

    XtAppAddActions(app_con, window_actions, XtNumber(window_actions));

    /* make the root entry, it is called "root" */
    root= new_entry( "root", 4);

    /* parse the input buffer into a tree rooted at this point */
    xtpanel_scan( buffer, root );

    /* create box within user's widget and put all objects in it */
    build_box( root, rootobj, 0 );

    /* clean up the parsed representation */
    delete_entry( root );

    return 0;
}

/* 
  This routine take the fully parsed object tree and makes the
  appropriate widgets from each object.
  */

/* this routine may be called recursively (if the user defined sub boxes ) */

void process_tree( root,parent )
     entry *root;
     Objdef *parent;
{
    entry* curr;
      
    /* loop over first level of tree */
    for( curr=root->child; curr != (entry*)0; curr=curr->next ){
	
	/* here is the big switch on item type */
	
	if( !strcmp( curr->tag, "window" ) ){
	    /* create the new window  and all objects in it */
	    build_window(curr, parent, 1);

	}else if( !strcmp( curr->tag, "box" ) || 
            !strcmp( curr->tag, "vbox") ||
            !strcmp( curr->tag, "hbox") ||
            !strcmp( curr->tag, "pane") ||
            !strcmp( curr->tag, "vpane") ||
            !strcmp( curr->tag, "hpane")){
	    /* create the new box and all objects in it */
	     build_box(curr,parent,curr->tag);
	    
	}else if( !strcmp( curr->tag, "form" ) ){
	    /* create the new form and all objects in it */
	    build_form(curr,parent,curr->tag);
	    
	}else if( !strcmp( curr->tag, "message" ) ){
	    build_message(curr,parent);
	    
	}else if( !strcmp( curr->tag, "button" ) ||
		 !strcmp( curr->tag, "toggle" )){
	    build_button(curr,parent,curr->tag);
	    
	}else if( !strcmp( curr->tag, "choice" ) ){
	    build_choice(curr,parent);
	    
	}else if( !strcmp( curr->tag, "slider" ) ||
		 !strcmp( curr->tag, "scrollbar" )){
	    build_slider(curr,parent,curr->tag);

	}else if( !strcmp( curr->tag, "graph" ) ){
	    build_graph(curr,parent);
	    
	}else if( !strcmp( curr->tag, "grid" ) ||
		 !strcmp( curr->tag, "togglegrid" )){
	    build_grid(curr,parent,curr->tag);
	    
	}else if( !strcmp( curr->tag, "menubutton" ) ){
	    build_menubutton(curr,parent);
	    
	}else if( !strcmp( curr->tag, "list" ) ){
	    build_list(curr,parent);
	    
	}else if( !strcmp( curr->tag, "dialog" ) ){
	    build_dialog(curr,parent);
	    
	}else if( !strcmp( curr->tag, "text" ) || 
	          !strcmp( curr->tag, "field" ) ){
	    build_text(curr,parent,curr->tag);
	    	    
	}else if( !strcmp( curr->tag, "var" ) ||
	          !strcmp( curr->tag, "startup" ) ){
	    build_variable(curr,parent,curr->tag);
	    
	}else if( !strcmp( curr->tag, "timer" ) ){
	    build_timer(curr,parent);
	
	}else if( !strcmp( curr->tag, "input" ) ||
		  !strcmp( curr->tag, "output" ) ||
		  !strcmp( curr->tag, "pipe" ) ){
	    build_input(curr,parent,curr->tag);
	
	}else if( !strcmp( curr->tag, "template" ) ){
	    save_template(curr);
	
	}else if( is_template( curr->tag )){
	    build_template(curr,parent, curr->tag );
	
            
	/* sciplot added 7.6.95, gdg. */

	}else if( !strcmp( curr->tag, "sciplot" ) ){
	    build_sciplot(curr, parent);
	}
	
    } /* end of loop over objects in tree */
    
}

/*
 * Function name: quit_xtpanel
 * Description: quits by destroying toplevel widget.
 * Arguments: none
 * Returns: none
 */
extern Widget toplevel;
void quit_callback() { quit_xtpanel( 0 ); }


void quit_xtpanel( ret )
int ret;
{
    XtDestroyWidget((Widget)toplevel);
    exit(ret);
}

void flush_display()
{
    XFlush( XtDisplay( toplevel ) );
}

