NONSEPAR [^ \t\n\{\}]
WHITE    [ \t\n]
ALPHA	 [A-Za-z]
ALPHANUM [A-Za-z0-9_-]
TAG	{ALPHA}{ALPHANUM}*=
TAGBRACE {ALPHA}{ALPHANUM}*=\{
SQ	\'([^']*\'\')*[^']*[']
DQ	\"([^"]*\"\")*[^"]*["]
BQ	\`([^`]*\`\`)*[^`]*[`]
LEFT    \{
RIGHT   \}

%{

/*
 * Copyright 1992 the Board of Trustees of the Leland Stanford Junior
 * University. Official permission to use this software is included in
 * the documentation. It authorizes you to use this file for any
 * non-commercial purpose, provided that this copyright notice is not
 * removed and that any modifications made to this file are commented
 * and dated in the style of the example below.
 */
 
/*
 *
 *  source file:   ./xtpanel/xtpanel-scan.l
 * 
 * Steve Cole, Dave Nichols (SEP), August 31 1992
 *      Inserted this sample edit history entry.
 *      Please log any further modifications made to this file:
 * Steve Cole, Dave Nichols (SEP), November 20 1992 -  version 2.00
 * 	1) added new objects: toggle, scrollbar, graph.
 * 	2) added new actions: ASSIGN, SET.
 * 	3) objects can have multiple actions.
 * 	4) backquoted strings in actions get executed at action time.
 * Steve Losen (scl@virginia.edu) 3 Mar 1993
 * 	introduced `` as an escaped backquote.
 * Dave Nichols (SEP), Sept 2 1993 -  
 *      added support for using flex
 * Dave Nichols (SEP), Nov 16 1993 -  
 *      Use X11/Xos.h, this requires undefining "input" before
 *	the include as the X11 includes have a variable called input.
 * Dave Nichols (Geco-Prakla), Sept 29 1995 -  
 *      Don't redefine yylook as a macro, gcc complains about later
 *      definition.
 */

/* lexical scanning for xtpanel */

/* #define DEBUG */

#undef input

#include <ctype.h>
#include <stdio.h>
#include <string.h>

#include <X11/Xos.h>

#if defined(SVR4) || defined(SYSV)
#define bcopy(a,b,c) memcpy(b,a,c)
#endif

#include "tree.h"
#include "eval_command.h"

#ifndef MIN
#define MIN(A,B) (A<B?A:B)
#endif
#ifndef MAX
#define MAX(A,B) (A>B?A:B)
#endif

#define MAX_NEST 20
static int stack_ptr = -1;
static char *input_buf;
static char *input_buf_start;
static char *input_buf_end;

static entry *entry_stack[MAX_NEST];

#ifdef FLEX_SCANNER

/* one character at a time to ensure accurate error reporting */

#undef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
   { if( input_buf == input_buf_end+1 ) { \
	result=0; \
     }else{ \
	result=1;\
        *buf = *input_buf;\
        input_buf++; \
     } \
   }

/*        result = MIN( max_size, input_buf_end-input_buf+1 ); \ */
/*        bcopy(input_buf,buf,result); \ */
/*        input_buf += result; \ */
/*     }\ */
/*   } */

#undef yywrap

#else

#undef input
#define input() ((int) *(input_buf++))
#undef unput
/* The redundant  =(c) insures side effects of expressions occur */
#define unput(c) (*(--(input_buf))=(c))


/* define the look ahead buffer size to something big
 * This is needed in case they have a very big text 
 * string in quotes.
 */
#undef YYLMAX
#define YYLMAX 50000

char my_yytmp[YYLMAX];

#define yywrap xtpanel_wrap
#define yylex  xtpanel_lexscan
#define yylook xtpanel_yylook

#endif

entry *tree_ent;
entry *treetop;
int len;
int left=0;

static void syntax_err();
static int massage();

extern int bad_obj_syntax();
extern int bad_attr_syntax();

/* template check */
extern void new_template();

void template_check( a, b ) 
entry* a, *b;
{
if( !strcmp(a->tag,"template") && !strcmp(b->tag,"name" )) 
  new_template( b->value );
}


%}
%S FOUNDTAG 
%%
<FOUNDTAG>{SQ}		{
			 len = yyleng-2; tree_ent->value=malloc(len+1);
			 len = massage(yytext+1,tree_ent->value,len,yytext[0]);
			 tree_ent->value[len]='\0';  
			 add_child( treetop, tree_ent );
			 template_check( treetop, tree_ent );
#ifdef DEBUG
			 fprintf(stderr,"<FOUNDTAG>{SQ}\n");
			 fprintf(stderr,"%s\n",tree_ent->value);
#endif
			 BEGIN 0; 
			 }
<FOUNDTAG>{BQ}		{ /* backquotes are now evaluated in get_value */
			 len=yyleng; tree_ent->value=malloc(len+1);
		 	 bcopy(yytext,tree_ent->value,len+1); 
			 tree_ent->value[len]='\0'; 
			 add_child( treetop, tree_ent );
			 template_check( treetop, tree_ent );
#ifdef DEBUG
			 fprintf(stderr,"<FOUNDTAG>{BQ}\n");
			 fprintf(stderr,"%s\n",tree_ent->value);
#endif
			 BEGIN 0; 
			 }
<FOUNDTAG>{DQ}		{
			 len = yyleng-2; tree_ent->value=malloc(len+1);
			 len = massage(yytext+1,tree_ent->value,len,yytext[0]);
			 tree_ent->value[len]='\0'; 
			 add_child( treetop, tree_ent );
			 template_check( treetop, tree_ent );
#ifdef DEBUG
			 fprintf(stderr,"<FOUNDTAG>{DQ}\n");
			 fprintf(stderr,"%s\n",tree_ent->value);
#endif
			 BEGIN 0; 
			 }
<FOUNDTAG>[^'"]{NONSEPAR}*	{
			 len=yyleng; tree_ent->value=malloc(len+1);
		 	 bcopy(yytext,tree_ent->value,len+1); 
			 tree_ent->value[len]='\0'; 
			 add_child( treetop, tree_ent );
			 template_check( treetop, tree_ent );
#ifdef DEBUG
			 fprintf(stderr,"<FOUNDTAG>{NONSEPAR}\n");
			 fprintf(stderr,"%s\n",tree_ent->value);
#endif
			 BEGIN 0; 
			 }
{TAGBRACE}             { /* start of a new level of braces */
			 tree_ent = new_entry( yytext, 
					get_taglen(yytext,yyleng-1));
			 /* check that this is a valid sublevel name */
			 if( bad_obj_syntax( treetop, tree_ent ) ){
			  syntax_err("Invalid tag name");
			 }
                         push_entry( tree_ent );
			 left++;
#ifdef DEBUG
			 fprintf(stderr,"{TAGBRACE}\n");
			 fprintf(stderr,"%s\n",tree_ent->tag);
#endif
		     	}
([ \t]){TAGBRACE}  	{ /* start of a new level of braces */
			 tree_ent = new_entry( yytext+1, 
					get_taglen(yytext+1,yyleng-2));
			 /* check that this is a valid sublevel name */
			 if( bad_obj_syntax( treetop, tree_ent ) ){
			  syntax_err("Invalid tag name");
			 }
			 push_entry(tree_ent);
			 left++;
#ifdef DEBUG
			 fprintf(stderr,"[ \\t]{TAGBRACE}\n");
			 fprintf(stderr,"%s\n",tree_ent->tag);
#endif
		     	}
{TAG}/{NONSEPAR}	{
			 tree_ent = new_entry( yytext, yyleng-1 );
			 /* check that this is a valid entry name */
			 if( bad_attr_syntax( treetop, tree_ent ) ){
			  syntax_err("Invalid tag name");
			 }
#ifdef DEBUG
			 fprintf(stderr,"{TAG}/{NONSEPAR}\n");
			 fprintf(stderr,"%s\n",tree_ent->tag);
#endif
                         BEGIN FOUNDTAG;
			 }
{TAG}{WHITE}		{ /* an empty tag */
			  syntax_err("unexpected empty field ");
			 }
{RIGHT}			{ /* end of a level of braces */
			 pop_entry();
			 left--; 
#ifdef DEBUG
			 fprintf(stderr,"{RIGHT}\n");
#endif
			 if( left<0 ) {
			    syntax_err("unmatched right brace");
			 }
			}
{LEFT}			{ /* found a left brace in the wrong context */
#ifdef DEBUG
			 fprintf(stderr,"{LEFT}\n");
#endif
			  syntax_err("left brace found in an unexpected place");
			}
{WHITE}+	        {
#ifdef DEBUG
			fprintf(stderr,"Ignored whitespace\n");
			fprintf(stderr,"length %d\n",yyleng);
#endif
			}
^\![^\n]*   		{/* skip comment lines */;
#ifdef DEBUG
			 fprintf(stderr,"Comment line\n");
#endif
			}
\![^\n]*			{ /* comment not at start of line, allow for now */
#ifdef DEBUG
			fprintf(stderr,"Trailing comment\n");
#endif
/*			syntax_err("comment introducer not at start of line");*/
			}
.			{
#ifdef DEBUG
			fprintf(stderr,"Something unrecognised\n");
#endif
			syntax_err("unrecognisable construct");
			}
%%


/* workhorse to decode xtpanel files; */

entry* xtpanel_scan( buffer, root )
char* buffer;
entry* root;
{
 extern int yylex();

#ifdef DEBUG
  fprintf(stderr,"xtpanel_scan\n%s\nxtpanel_scan_end\n",buffer);
#endif
 

 /* note the input buffer should be null terminated */
 input_buf = buffer;
 input_buf_start = input_buf;
 input_buf_end = input_buf + strlen(input_buf)-1 ;

 /* all the info we find will be stored under the root entry */
 push_entry( root );

#ifdef FLEX_SCANNER
 yy_init=1;
#endif

 yylex();
 if( left > 0 ) {
    fprintf(stderr," %d unmatched left braces present \n",left);
    exit(-1);
 }
 if( left < 0 ) {
    fprintf(stderr," %d unmatched right braces present \n",-left);
    exit(-1);
 }

}
/* get the tag length from a string that may have spaces around the equal */
int get_taglen( str, len )
char *str; int len;
{
char *temp;
int ret;
temp = malloc( len+1); strncpy( temp, str, len ); temp[len]='\0';
ret= strcspn( temp, " =" ) ;
free(temp);
return ret;
}


/* print a syntax error, try and give them enough context to figure
 * out where it is */
static void  syntax_err( message )
char *message;
{
    char temp[41];
    int lchar,rchar,nchar,i,j,linepos;

    /* print the error message */
    fprintf(stderr,"\nSyntax Error: %s\n\n", message);

    /* back up to the character causing the error */
    input_buf--; 

    /* print enough characters on either side to give he user some context*/
    lchar = MIN( 20, input_buf - input_buf_start );
    rchar = MIN( 20, input_buf_end - input_buf);
    nchar = lchar+rchar+1;
    strncpy( temp, (input_buf-lchar), nchar ); temp[nchar] = '\0';

    /* print stuff to the left of the point */
    for( i=0; i<lchar; i++ ){
        putc( temp[i], stderr );
	if( temp[i] == '\n' ){
	   linepos=0;
        }else if( temp[i] == '\t' ){
	   linepos += 8;
        }else{
           linepos++;
        }
    }

    /* print from the point up to the end or the next newline */
    for( i=lchar; ( i<nchar && temp[i] != '\n' ) ; i++ ){
        putc( temp[i], stderr );
    }

    /* print the marker that points at the error */
    putc( '\n', stderr );
    for( j=0; j<linepos; j++ ){ putc( ' ', stderr ); }
    fprintf(stderr,"^--- error detected at text ending here (I think)\n",temp);

    /* print the rest of the text (if any ) */
    for( j=i+1; j<nchar; j++ ){  putc( temp[j], stderr ); }
    putc( '\n', stderr );

    exit(-1);
}

push_entry( ent )
entry* ent;
{
if( stack_ptr >=0 ) add_child( entry_stack[stack_ptr], ent );
stack_ptr++;
if( stack_ptr >=MAX_NEST ){
   fprintf(stderr," items nested too deeply, only %d levels supported\n",
            MAX_NEST);
   exit(-1);
}
entry_stack[stack_ptr] = ent;
treetop = entry_stack[stack_ptr];
}


pop_entry()
{
stack_ptr--;
if( stack_ptr >= 0 ) treetop = entry_stack[stack_ptr] ;
}

int
yywrap()
{
    pop_entry();
    if(stack_ptr < 0) return(1);
    return(0);
} 


/* play with quoted strings and newlines etc. */
static int
massage(string,out,len,quote)
register char *string, *out;
register int len, quote;
{
  register int i,j;
  for(i=0,j=0; i<len-1; j++) {
    out[j]=string[i++];

    if(out[j]==quote) /* compress doubled quotes */
      if(string[i]==quote) i++;
    
    if(out[j]=='\\') {
      if(string[i]== 'n') { 
	   /* turn backslash n "\n" into a newline */
           i++; out[j] = '\n';
      }else if( string[i] == '\n' ){ /* remove backslash newline */
	   i++; j--;
      }
    }
    
  }
  if(i<len) out[j++] = string[i];
  return(j);
}
